Search.additionalData = [
["history.html","Revision History",
"Revision History\n",0]
,["history.html","Version 1.0 (2014/08/05)",
"Version 1.0 (2014/08/05)\n",0]
,["history.html","Additions and Changes",
"Additions and Changes\nCONFIDENTIAL\n",0]
,["title.html","3DS Performance Tips",
"3DS Performance Tips\nVersion 1.0\n\tNintendo Confidential\n\tThe content of this document is highly confidential and should be handled accordingly.\nThe content of this document cannot be duplicated, copied, reprinted, transferred, distributed, or loaned in whole or in part without the prior approval of Nintendo.\nThis document contains confidential and proprietary information of Nintendo and is also protected under the copyright laws of the United States and foreign countries.No part of this document may be released, distributed, transmitted or reproduced in any form or by any electronic or mechanical means, including information storage and retrieval systems, without permission in writing from Nintendo.\n© 2014 Nintendo. All rights reserved.\nCTR-06-0010-002-A\nAll company and product names in this document are the trademarks or registered trademarks of their respective companies.\n",0]
,["Pages/Page_55591963.html","1. Introduction",
"1. Introduction\n",0]
,["Pages/Page_55591963.html#PageId_55591964","1.1. About This Document",
"1.1. About This Document\nThis document introduces efficient methods for implementing graphics operations and summarizes certain aspects of Nintendo 3DS Family ( hereafter \"3DS\") game programming that warrant special attention.\nThis document is intended for an audience that has some prior knowledge of 3DS programming because it contains highly technical tips about programs and graphics operations.\nFor more information about 3DS programming specifications, see the manuals included with the CTR-SDK and the Function Reference Manual.\n Note:\n This document was written for CTR-SDK 9.12.x. If you are using a different version of the SDK, the methods described here may not significantly improve performance. Always refer to the most recent version of the documentation.\n",0]
,["Pages/Page_55591963.html#PageId_55591965","1.2. Items Tagged (Important)",
"1.2. Items Tagged (Important)\nThis document uses (Important) to tag any items that have a significant impact on performance. If you do not handle these items, they may become bottlenecks and cause a large performance drop.\nTo achieve acceptable performance on a 3DS system, we strongly recommend addressing every item that has been tagged (Important).\n",0]
,["Pages/Page_55591963.html#PageId_55591966","1.3. About the CPU Cores",
"1.3. About the CPU Cores\nOne of the cores is made available as the application core for exclusive use by game applications, while the remaining core is made available as the system core for use by the system.\nVarious processes run on the system core under strict scheduling to ensure overall system stability. Applications can only use part of the system core due to the extreme difficulty of developing processes that run on the system core from an application, and because performance improvements do not necessarily result from doing so.\nFor more information about the system core, see Chapter 7. Processing Handled by the System Core.\n",0]
,["Pages/Page_55591963.html#PageId_56758243","1.4. Platform Notation",
"1.4. Platform Notation\nThis documentation uses the platform name when specifying a particular device in the Nintendo 3DS Family.\nThe platform names and the corresponding devices are listed below.\n Table 1-1. Platform Names and Corresponding Devices\n \tName \t(TBD) \tNintendo 3DS \tNintendo 3DS XL \tNintendo 2DS \tComments\n \t3DS \tYes \tYes \tYes \tYes \tRefers to all platforms.\n \tSNAKE \tYes \t \t \t \tRefers to SNAKE only.\n \tCTR \t \tYes \tYes \tYes \tRefers to CTR, SPR, and FTR. However, when SPR or FTR is also listed, this refers only to CTR.\n \tSPR \t \t \tYes \t \tRefers to SPR only.\n \tFTR \t \t \t \tYes \tRefers to FTR only.\nCONFIDENTIAL\n",0]
,["Pages/Page_55591967.html","3. Graphics Processing",
"3. Graphics Processing\nThis chapter provides information about graphics processing.\nGraphics processing is divided between the CPU and GPU. To optimize system performance, you must determine whether CPU or GPU processing presents a bottleneck and take appropriate measures to deal with it.\n",0]
,["Pages/Page_55591967.html#PageId_55591968","3.1. About the Graphics Library (Important)",
"3.1. About the Graphics Library (Important)\nWhen using 3DS graphics, 3D commands and instructions to the GPU called command requests are put on a command list and the GPU operates based on these commands. The graphics library can be divided into two parts: the part that controls command requests, and the part that actually generates commands for rendering. The nngx library is used for controlling the command list and generating general commands.\nThe following four libraries are used to generate 3D commands for actual rendering.\n \tGR Library \tNW4C \tGD Library \tGL Library (DMPGL) (not recommended)\nIn addition to using these libraries, you can use a method of directly issuing commands from the application. The following sections detail the various methods of generating commands. Please use the library best suited to your purpose.\n",0]
,["Pages/Page_55591967.html#PageId_55591969","3.1.1. GR Library",
"3.1.1. GR Library\nThis graphics library is used to assist the direct creation of 3D commands. Because no status management is carried out inside this library, it executes faster than other graphics libraries, and also uses less memory. To use this library, you must have an in-depth understanding of how each GPU register is set (described in the CTR Programming Manual: Advanced Graphics). This library is also hard to use because the user must handle detailed processing such as error checking.\n",0]
,["Pages/Page_55591967.html#PageId_55591970","3.1.2. NW4C (NintendoWare for CTR)",
"3.1.2. NW4C (NintendoWare for CTR)\nCombining both sound and graphics capabilities, NW4C is middleware provided by Nintendo to aid in application development. The graphics capabilities of NW4C generate direct commands without using other graphics libraries. In addition, libraries have been optimized to provide full performance when implemented using NintendoWare. To optimize performance, we recommend using NintendoWare.\n",0]
,["Pages/Page_55591967.html#PageId_55591971","3.1.3. GD Library",
"3.1.3. GD Library\nThis graphics library has a similar structure to general 3D library APIs. The API has an independent format, but no understanding of the individual GPU registers is required in order to use GD. Nintendo recommends using GD if you intend not to use NW4C and plan to perform graphics processing without learning information about GPU registers.\n",0]
,["Pages/Page_55591967.html#PageId_55591972","3.1.4. GL Library (DMPGL)",
"3.1.4. GL Library (DMPGL)\nThis library provides a rich range of functionality, including API functions based on OpenGL ES, error handling, and state-difference management. However, compared to the other libraries, the processes of the GL library put a heavy load on the CPU because the library issues many redundant commands.\n Note:\n Use of this library is not recommended because CPU performance is greatly reduced.\n",0]
,["Pages/Page_55591967.html#PageId_55591973","3.1.5. Direct Creation of 3D Commands",
"3.1.5. Direct Creation of 3D Commands\nThis method directly creates register write commands to be applied to the GPU. Fast operations can be achieved by creating only those commands that are necessary. However, this method is the most difficult way to implement graphics because it requires an in-depth understanding and detailed knowledge of how to set each GPU register, as described in the CTR Programming Manual: Advanced Graphics. We recommend using the GR library, which is implemented in a way that assists direct creation of commands to be issued to the GPU.\n",0]
,["Pages/Page_55591967.html#PageId_55591974","3.2. Command Lists",
"3.2. Command Lists\nWhen the 3DS system performs a graphics operation, the CPU generates rendering instructions (3D commands, command requests) and the GPU processes them. The command list consists of command requests, which are the instructions that the CPU conveys to the GPU, in addition to 3D commands, which are instructions that the GPU references directly.\nWhen graphics are executed, the CPU conveys instructions to the GPU based on command requests, and the GPU performs various operations in accordance with the content of those command requests. If the command request received from the CPU is a render command request, the GPU directly references the 3D command buffer for rendering.\nFor more information about the functions that issue command requests, see Section 8.5.10. Functions That Issue Command Requests. The CPU load for processing command requests and the GPU wait-time both increase with the number of command requests processed during rendering.\n Note:\n Because command requests are primarily processed by the system core, the execution of large volumes of command requests can affect the processing efficiency of the system core.\n For more information about the processing performed by the system core, see Chapter 7. Processing Handled by the System Core.\n",0]
,["Pages/Page_55591967.html#PageId_55591975","3.2.1. About Issuing Commands (Important)",
"3.2.1. About Issuing Commands (Important)\nYou can significantly reduce CPU usage by issuing only those commands that are necessary to the command list without using the DMPGL.\n Note:\n For more information about command specifications, see the CTR Programming Manual: Advanced Graphics.\n Note:\n The implementation of NW4C directly issues only the commands that are needed to improve performance. Also, the GR library that is used to assist in directly issuing commands is included in the CTR-SDK.\n Related Functions:\n nngxAdd3DCommand\n",0]
,["Pages/Page_55591967.html#PageId_55591976","3.2.2. Double-Buffering Command Lists (Important)",
"3.2.2. Double-Buffering Command Lists (Important)\nWhen there is a single command list, you must wait until the GPU has finished processing, and then clear the command list and generate the render command for the next frame. You can make the CPU and GPU run in parallel by double-buffering the command list. With a double-buffered approach, the CPU issues rendering instructions to one of the command lists while the GPU processes the other command list.\n Note:\n If double-buffering a command list, the frame for which graphics rendering is actually being performed is delayed one frame from the frame for which commands accumulate.\n For a sample implementation that demonstrates how to double-buffer a command list, see SampleDemos/gx/Api/CommandListDouble.\n Related Functions:\n nngxBindCmdList, nngxRunCmdList\n",0]
,["Pages/Page_55591967.html#PageId_55591977","3.2.3. Reusing 3D Commands (Important)",
"3.2.3. Reusing 3D Commands (Important)\nYou can lighten the processing load required of the CPU for issuing 3D commands by reusing 3D commands that have already been generated. This is particularly effective for the frequently used commands for setting vertex shaders and lookup tables, and for the 3D commands for the overall material settings included in them.\n",0]
,["Pages/Page_55591967.html#PageId_55591978","3.2.4. Command Caches",
"3.2.4. Command Caches\nThe command cache feature is provided to create command lists in advance for repeated use. By using a command cache, you can reduce the number of duplicate commands that are issued and the processing load on the CPU.\nThere are two ways to use a command cache. One is to copy and add command buffers. The other is to add command requests that reference command buffers.\n Note:\n For more information about command caches, see the CTR Programming Manual: Advanced Graphics. For more information about how to implement them, see SampleDemos/gx/Api/CommandCacheSimple.\n Related Functions:\n nngxStartCmdlistSave, nngxStopCmdlistSave\n",0]
,["Pages/Page_55591967.html#PageId_55591979","3.2.5. Using Command Buffer Jumps (Important)",
"3.2.5. Using Command Buffer Jumps (Important)\nWith a command buffer jump, the next command buffer execution address that the GPU references can be moved based on an instruction (3D command) in the command buffer. By using this feature you can make the GPU reference a series of command buffers without incurring any command request processing by the CPU.\nThe following functions have been prepared for use of this command buffer jump feature.\n \t nngxAddJumpCommand\nUse this function to make a jump from the current command buffer to another command buffer.\n \t nngxAddSubRoutineCommand\nUse this function to call another command buffer from the current command buffer as a subroutine.\nCompared to copying and adding command caches, using command buffer jumps can significantly reduce the processing load placed on the CPU that normally results from the copying of command buffers. It can also significantly reduce the number of command requests compared to the number added when referencing command caches. The result is both a lighter processing load on the system core and less waiting by the GPU for the command request processing of the CPU.\nThe reloading of 3D commands involves a slightly higher cost to the GPU compared to simply executing a series of command buffers. But because processing speed can slow due to excessive command buffer jumping, we recommend using the feature for only limited groups of 3D commands.\n Note:\n For more information about the command execution registers involved in command buffer jumping, see the CTR Programming Manual: Advanced Graphics.\n Related Functions:\n nngxAddJumpCommand, nngxAddSubRoutineCommand\n",0]
,["Pages/Page_55591967.html#PageId_55591980","3.2.5.1. Subroutine Placement",
"3.2.5.1. Subroutine Placement\nIf you plan to call a command buffer as a subroutine, placing the subroutine in VRAM may improve processing speed when the GPU accesses the command buffer, which leads to an overall reduction in GPU processing time.\nWhen commands that could potentially cause frequent access to the command buffer are placed in VRAM, the bottleneck shifts from the command buffer to actual command processing. Accordingly, if access to the command buffer is not a bottleneck (this depends on the implementation of the application), little improvement can be expected from using this technique.\nIn the GPU rendering pipeline, commands to modules that are upstream from rasterization in the pipeline are processed using one cycle per command. When many commands to those upstream modules are made, command buffer access is typically more of a bottleneck than command processing. (See the \"3D Command Execution Cost\" section in the CTR Programming Manual: Advanced Graphics.)\nAccordingly, placing commands that load shaders in VRAM could significantly reduce GPU processing time.\nConversely, commands to the rasterization module or modules that are downstream from rasterization in the pipeline are processed using two cycles per command. When there are many commands to those downstream modules, command processing is typically more of a bottleneck than command buffer access.\nThe size of the command buffer also affects the bottlenecking tendencies described above. The larger the command buffer, the more likely command buffer access is the bottleneck because memory must be accessed more frequently. Command buffer size is affected by whether you use single commands (header + one piece of data) or burst commands (header + multiple pieces of data), even if the number of commands is the same in both cases.\nAlso, if a command buffer being called as a subroutine is placed in device memory, the cost of switching when executing that subroutine is added directly to the GPU processing time.\n",0]
,["Pages/Page_55591967.html#PageId_55591981","3.2.6. Reusing Commands for Stereoscopic Display (Important)",
"3.2.6. Reusing Commands for Stereoscopic Display (Important)\nTo render a scene stereoscopically, you must render it twice with only the camera position changed.\nBy saving the commands from the first rendering pass in a command cache (or by some other means) and then reusing all of them—except for camera-related settings—in the second rendering pass, you can dramatically reduce the number of commands that must be issued for rendering.\n Note:\n Fragment lights are set in camera coordinates. To represent lighting precisely, you must convert fragment lights into each camera’s coordinate system, which means you must also change commands related to fragment lights.\n",0]
,["Pages/Page_55591967.html#PageId_55591982","3.3. Vertex Shader Programs and Lookup Tables",
"3.3. Vertex Shader Programs and Lookup Tables\n",0]
,["Pages/Page_55591967.html#PageId_55591983","3.3.1. Vertex Shader Programs",
"3.3.1. Vertex Shader Programs\nBecause vertex shader processing is run on each vertex, we recommend that you reduce the number of vertex lights, branch instructions, and so on as much as possible. Care is required because the generation of 3D commands that switch (load) vertex shaders places a load on the CPU. At runtime, shader switching also places a load on the GPU.\nTo reduce the number of times a vertex shader is loaded, take the following steps.\n \tManage the rendering order to minimize the number of times that vertex shaders are switched. \tPack multiple shader programs into a single binary. This way, you can switch shaders by simply changing the shader entry point without loading a binary.\n",0]
,["Pages/Page_55591967.html#PageId_55591984","3.3.2. Reducing Lookup Table (LUT) Loads",
"3.3.2. Reducing Lookup Table (LUT) Loads\nWhen you configure lookup tables, the CPU issues the lookup tables themselves as commands to the GPU. As a result, there is a CPU cost associated with switching lookup tables. Because there are registers for each lookup table type (such as D0, D1, and SP), we recommend that you avoid switching the lookup table whenever possible.\nWhen two materials are each stored in separate LUTs, for example, you could set one LUT in D0 and one in D1, and then turn them on and off for the corresponding materials.\n",0]
,["Pages/Page_55591967.html#PageId_55591985","3.3.3. Changes in Loading Speed Due to 3D Command Buffer Address and Size",
"3.3.3. Changes in Loading Speed Due to 3D Command Buffer Address and Size\nThe address and size of the 3D command buffer may affect loading speed during execution.\nTo execute the 3D command buffer, use one of the following two methods.\n \tExecution using 3D executable commands\nLoading speed is affected by the size of data as measured between the addresses of consecutive delimiter commands added using nngxFlush3DCommand or nngxSplitDrawCmdlist.\n \tExecution using a register that executes a command buffer (command buffer jump)\nLoading speed is affected by the address and size of the command buffer added using nngxAddJumpCommand or by a command buffer executed as a subroutine or executed to return to the calling processes from a subroutine added using nngxAddSubroutineCommand.\nMore efficient transfer rates may result when using the following buffer addresses and sizes.\n \tThe address has 128-byte alignment and the size of data is a multiple of 256 bytes (256 bytes, 512 bytes, 768 bytes, and so on.) \tThe address is not 128-byte aligned, but the size as measured from the immediately previous address with 128-byte alignment to the end of the 3D command buffer is a multiple of 256 bytes.\n For example, if the address and size of the 3D command buffer are 0x20000010 and 0x1F0, respectively, the immediately previous address with 128-byte alignment is 0x20000000 just 0x10 before. The data size from there to the end of the buffer is 0x1F0 + 0x10, or 0x200, which is a multiple of a 256.\nAlthough the address and size of the 3D command buffer can influence loading speed as described above due to the way the GPU is implemented, you may not see large benefits due to factors such as the location of the buffer, the content of 3D commands, and memory access conflicts with other modules.\n",0]
,["Pages/Page_55591967.html#PageId_55591986","3.4. Vertex Buffers and Texture Buffers",
"3.4. Vertex Buffers and Texture Buffers\n",0]
,["Pages/Page_55591967.html#PageId_55591987","3.4.1. Placement of Vertex Buffers and Texture Buffers (Important)",
"3.4.1. Placement of Vertex Buffers and Texture Buffers (Important)\nVertex buffers, vertex index buffers, and texture buffers can be placed in main memory or VRAM. Because it is more convenient for the GPU to access VRAM than main memory, you can place buffers in VRAM to improve performance.\nWith the default settings, the GPU is given priority when there is a main memory access conflict between the CPU and the GPU. If the CPU is executing a process that heavily depends on memory access, this can cause a precipitous drop in performance. To reduce the processing load on the CPU, place data that is directly accessed by the GPU into VRAM instead of main memory.\n Note:\n Use nn::gx::SetMemAccessPrioMode to configure main memory access priorities for the CPU and the GPU.\nFor information about the structure of the CPU, the GPU, and the various memory devices, see Section 8.3. Hardware Configuration.\nIf vertex attributes and textures (multi-textures) to be used simultaneously are placed in both VRAM and main memory, main memory becomes a bottleneck on access speed. All buffers that are used simultaneously must therefore be placed in VRAM.\nPlacing multi-textures in different VRAMs may lower performance. We recommend placing textures that are used simultaneously in the same VRAM.\n",0]
,["Pages/Page_55591967.html#PageId_55591988","3.4.2. Effective Use of the Vertex Cache (Important)",
"3.4.2. Effective Use of the Vertex Cache (Important)\nThe 3DS system has a vertex cache that can temporarily cache processed vertex data using its vertex index as a key, and then use that cache data later for the same input vertex.\nThe vertex cache is only effective in rendering primitives when glDrawElements (or an equivalent 3D command) uses a vertex buffer for rendering. Up to 32 entries can be cached in the vertex cache. If more than 32 entries are input, the existing entries are overwritten, starting with the least-recently accessed entries. The cache is flushed each time glDrawElements is called.\n",0]
,["Pages/Page_55591967.html#PageId_55591989","3.4.3. Improving Efficiency of Vertex Indices",
"3.4.3. Improving Efficiency of Vertex Indices\nWhen specifying vertex indices, rendering efficiency can be improved by using the vertex cache to its fullest.\nIn the case of similar vertex indices, the efficiency of GL_TRIANGLES is no worse than that of GL_TRIANGLE_STRIP. In some cases, rendering with GL_TRIANGLES results in better efficiency because adjustments are sometimes necessary when using GL_TRIANGLE_STRIP with degenerate polygons. Using optimized vertex indices, GL_TRIANGLES allows for the most efficient rendering.\n",0]
,["Pages/Page_55591967.html#PageId_55591990","3.4.4. Updating Buffers",
"3.4.4. Updating Buffers\nWhen vertex and texture data are updated, the GPU accesses data in main memory and thus the CPU internally applies the changes to the GPU. Because this process puts a relatively large overhead on the system, the overall processor load will increase proportionally to the number of calls to such functions.\nA single command request is issued with each transfer for buffers placed in VRAM; this also increases the CPU load.\nWe recommend that you call vertex and buffer update functions in advance when the application is being initialized, load vertex buffer and texture data early, avoid loading data during per-frame operations whenever possible, and transfer data all at once.\n",0]
,["Pages/Page_55591967.html#PageId_55591991","3.4.5. Using Interleaved Arrays",
"3.4.5. Using Interleaved Arrays\nWhen rendering a model, it is more efficient to access memory using vertex buffers as a single interleaved array that combines vertex attributes rather than as separate arrays for each vertex attribute. This is particularly relevant when vertex arrays are placed in main memory because then the CPU and GPU compete over memory access. You can minimize the adverse effect on CPU performance by making memory access more efficient.\n Note:\n Beginning with NW4C 1.1.0, an interleaved array is used for the vertex buffer.\n",0]
,["Pages/Page_55591967.html#PageId_55591992","3.4.6. Reducing Texture Sizes",
"3.4.6. Reducing Texture Sizes\nThe GPU has L1 and L2 texture caches. These cache regions one at a time; by using smaller textures, you can suppress the number of texture fetches. We also recommend that you use smaller textures because it reduces VRAM usage.\n Note:\n Each texture unit has its own L1 texture cache, which holds 256 bytes.\n The L2 texture cache holds 8 KB and is shared by all texture units.\n",0]
,["Pages/Page_55591967.html#PageId_55591993","3.4.7. Using Mipmaps (Important)",
"3.4.7. Using Mipmaps (Important)\nWhen you shrink and display a texture, there are many texture cache misses and GPU performance drops. If mipmaps are used, however, appropriately sized textures are accessed to shrink textures. Use mipmaps for any textures that you expect to shrink onscreen.\nConversely, you can conserve VRAM by not using mipmaps for textures that are guaranteed to always be displayed at a fixed size.\n",0]
,["Pages/Page_55591967.html#PageId_55591994","3.4.8. Using the ETC1 Texture Format (Important)",
"3.4.8. Using the ETC1 Texture Format (Important)\nThe ETC1 texture format reduces the per-texel data size by 4 bits. In addition to decreasing VRAM usage, a reduction in texture size has the advantage of economizing the memory ranges that are accessed. A texture cache that only uses the ETC1 format is treated as being compressed and is advantageous in terms of cache efficiency.\nWe recommend using the ETC1 format whenever possible.\n Note:\n Texture formats other than ETC1—including ETC1A4—are expanded to the (32-bit) RGBA8 format in the cache.\n",0]
,["Pages/Page_55591967.html#PageId_55591995","3.4.9. Texture Image Layout Orientation (Important)",
"3.4.9. Texture Image Layout Orientation (Important)\n You can achieve better texture cache hit rates by aligning the orientation of the texture image used to the order in which fragments are generated in the framebuffer (Layout Pattern 2 in Figure 2-2) instead of aligning to the LCD display orientation (Layout Pattern 1 in Figure 2-2). This can improve display performance, particularly for 2D backgrounds and characters. The normal LCD layout orientation is shown in Figure 2-1, below.\n Figure 3-1. Scan Line Orientation and Fragment Generation Order\nThe content of the framebuffer copied to the display buffer is what is actually shown in the LCD, but the important point to note here is the relationship between the fragment generation order and the texture image pixel order. The system rasterizes images in 8×8 pixel blocks (the blue box in Figure 2-1), but it fetches textures in 8×4 texel blocks. The green box in Figure 2-1 shows the orientation of the texture block fetched in Layout Pattern 1, and the red box shows the orientation for Layout Pattern 2.\n Figure 3-2 shows the two texture image layout patterns. Because texture formats are handled in 8×8 pixel blocks, data ordering in memory is either a1 → a2 → b1 → b2 → for Layout Pattern 1, or C1 → C2 → A1 → A2 → for Layout Pattern 2. In other words, when rasterizing the block shown in Figure 3-1, the blocks fetched in Layout Pattern 1 would be c2 → c1 → c2 → c1, whereas in Layout Pattern 2 the blocks fetched would just be C1 → C2. Pattern 2 requires fewer block switches, and the next block to rasterize is the very next texture block in memory.\n Figure 3-2. Texture Image Positioning Pattern\nThe yellow lines indicate the switching order for block fetching when two blocks are rasterized.\n",0]
,["Pages/Page_55591967.html#PageId_55594786","3.4.10. Differences Between nngxUpdateBuffer and nngxUpdateBufferLight (Important)",
"3.4.10. Differences Between nngxUpdateBuffer and nngxUpdateBufferLight (Important)\nWhen the CPU changes vertex or texture data that resides on device memory directly accessed by the GPU, the application must guarantee the consistency of this data. In other words, if the CPU manipulates (decompresses, copies, or changes) this data, it must apply those changes to the GPU side.\nThe nngxUpdateBuffer takes longer to execute in extended applications than in standard applications. For this reason, Nintendo recommends calling the nngxUpdateBufferLight function instead of nngxUpdateBuffer when you are only applying data manipulated by the CPU to the GPU. However, you must call the nngxUpdateBuffer function to access GPU rendering results with the CPU.\n Because calls to nngxUpdateBuffer entail some overhead, calling it multiple times might lead to a drop in performance. If you manipulate multiple pieces of data with the CPU, do not call the function for each such piece of data. Instead, call it once for the entire device memory region when all your CPU data operations are finished. This reduces the processing load.\n Note:\n Using the FS library to decompress data on device memory counts as manipulation of this data by the CPU.\n Calling the nngxUpdateBuffer function twice on a small region and calling it once on the entire device memory region have roughly equivalent costs.\n",0]
,["Pages/Page_55591967.html#PageId_55594792","3.4.10.1. The nngxAddVramDmaCommand and nngxAddVramDmaCommandNoCacheFlush Functions",
"3.4.10.1. The nngxAddVramDmaCommand and nngxAddVramDmaCommandNoCacheFlush Functions\nThe application must also take data consistency into account when transferring data from main memory to VRAM. In general, if the application uses DMPGL or the nngxAddVramDmaCommand function to transfer the data, the data is cached automatically by the function.\nIn contrast, if the application uses the nngxAddVramDmaCommandNoCacheFlush function to transfer data, the data is not cached by the function. When you transfer data from multiple regions, you can reduce the processing load by first using the nngxUpdateBufferLight function so that the application ensures data consistency, and then using the nngxAddVramDmaCommandNoCacheFlush function to transfer the data.\n",0]
,["Pages/Page_55591967.html#PageId_55591998","3.4.11. Vertex Data Loading and Vertex Shader Processing",
"3.4.11. Vertex Data Loading and Vertex Shader Processing\nIf no geometry shader is in use, loading vertex data under ideal memory access conditions could hide up to 52 cycles of vertex shader processing underneath the loading of each individual vertex of vertex data, regardless of whether the vertex data is placed in device memory or VRAM\nThis number of cycles is a theoretical value. The actual value can be reduced by several factors: differences in data size, the number of load arrays, changes in memory access speed due to other processes, and whether different types are combined in the vertex data attributes.\n",0]
,["Pages/Page_55591967.html#PageId_55591999","3.4.12. Effect of Combinations of Vertex Attribute on the Vertex Data Transfer Rate",
"3.4.12. Effect of Combinations of Vertex Attribute on the Vertex Data Transfer Rate\nDepending on the combination of type and size of vertex attribute data making up the load array, look-ahead transfer can increase the vertex data transfer rate when using a vertex buffer.\nLook-ahead transfer is used if the following conditions are satisfied.\n(\"The number of attributes of other than GL_FLOAT type\" + \"The number of attributes with a data length of 1\")\n<= (\"The number of attributes of GL_FLOAT type with a data length of 4\" + \"The number of attributes of GL_FLOAT type with a data length of 3\" / 2)\nThe data size of \"the number of attributes of other than GL_FLOAT type\" and the data type of \"the number of attributes with a data length of 1\" do not matter. Vertex attributes that meet more than one condition at the same time are counted for each condition category they fulfill. For example, a vertex attribute of GL_BYTE type with a data length of 1 is included in the count for \"the number of attributes of other than GL_FLOAT type\" and \"the number of attributes with a data length of 1.\"\nIf the two sides of the comparison for whether to perform look-ahead transfer are equal, the determination depends on the amount of data per load array. The less data there is, the faster the transfer rate. If the amount of vertex data is the same in both cases, the determination depends on the number of attributes in the load array. The fewer number of attributes in the load array, the faster the transfer rate.\n",0]
,["Pages/Page_55591967.html#PageId_55592000","3.4.13. Address Alignment of the Vertex Array",
"3.4.13. Address Alignment of the Vertex Array\nIt may be possible when using a vertex buffer to increase the efficiency of vertex array transfer during rendering by using a vertex array with 32-byte address alignment. The concept of a vertex array address as used here means the address obtained by adding the offset specified by glVertexAttribPointer (the value given by ptr) to the vertex buffer address.\nIt may not always be possible to improve transfer efficiency, because how much the transfer rate increases as compared to using a vertex array with an address having other than 32-byte alignment depends on the type and size of the vertex attribute, the location the vertex array is stored, and the content of the vertex index. Even if transfer performance is improved, it may not necessarily lead to better overall system performance unless vertex array transfer is the bottleneck.\n",0]
,["Pages/Page_55591967.html#PageId_55592001","3.5. Fragment Shaders",
"3.5. Fragment Shaders\n",0]
,["Pages/Page_55591967.html#PageId_55592002","3.5.1. Number of Fragment Lights",
"3.5.1. Number of Fragment Lights\nThe number of fragment lights affects GPU fill processing. The processing load increases proportionally with respect to both the fill area size and the number of lights. Avoid using fragment lighting when possible. For example, you can use static lighting and bake colors into vertices instead of sending normal data.\n Note:\n The GPU specifications always enable at least one fragment light. If you do not use the fragment lighting results, the per-fragment processing load does not change but you can reduce processing for the vertex shader’s quaternion output.\n",0]
,["Pages/Page_55591967.html#PageId_55592003","3.5.2. Layer Configuration (Important)",
"3.5.2. Layer Configuration (Important)\nChoose layer configurations with smaller numbers (less processing) whenever possible.\nIf you are only using the D0 LUT, for example, layer configuration 7 is more expensive than layer configuration 1 by two clock cycles times the number of fragment lights for each fill area.\n",0]
,["Pages/Page_55591967.html#PageId_55592004","3.5.3. Unused Texture Unit Settings (Important)",
"3.5.3. Unused Texture Unit Settings (Important)\nWhen a texture is set in a texture unit, it is fetched even if (for example) it is not used by a texture combiner. To avoid unnecessary memory access, do not configure a texture unit that is not accessed by the combiners.\n Note:\n Accessing memory to fetch unnecessary textures is detrimental to power consumption.\n",0]
,["Pages/Page_55591967.html#PageId_55592005","3.5.4. Interpolation Between Mipmap Levels",
"3.5.4. Interpolation Between Mipmap Levels\nWhen you configure linear interpolation to be used between mipmap levels, the number of texture fetches increases. Because this increases the GPU load and also leads to access conflicts, choose “nearest” interpolation between mipmap levels whenever possible.\n",0]
,["Pages/Page_55591967.html#PageId_55592006","3.5.5. Using or Not Using Blend Operations",
"3.5.5. Using or Not Using Blend Operations\nWithout blending, the framebuffer is only accessed for lights; with blending, read accesses also occur. Setting blend operations when they are not needed entails unnecessary memory accesses. Configure blend operations appropriately.\n",0]
,["Pages/Page_55591967.html#PageId_55592007","3.6. Framebuffers and Display Buffers",
"3.6. Framebuffers and Display Buffers\n",0]
,["Pages/Page_55591967.html#PageId_55592008","3.6.1. Avoiding the Use of glClear",
"3.6.1. Avoiding the Use of glClear\n Each time you use glClear to clear the framebuffer, two command requests are added, resulting in a large CPU load.\n Instead of using glClear, you can render a background model first with depth tests disabled and depth writes enabled to eliminate the framebuffer clear operation and save CPU/GPU processing.\nFor example, if you blend a polygon that covers the entire screen at the end of rendering to change the rendered brightness, the settings described above allow you to skip the clear operation for the depth buffer during the next rendering pass.\n Note:\n Settings that combine no depth testing with writing depth data cannot be made when using DMPGL. A library such as GR must be used instead to issue these commands.\n",0]
,["Pages/Page_55591967.html#PageId_55592009","3.6.2. Render Buffer Location (Important)",
"3.6.2. Render Buffer Location (Important)\nYou can place the color buffer and depth buffer in different locations in VRAM (VRAM-A or VRAM-B) to parallelize fill operations when the buffers are used simultaneously. This improves performance.\n",0]
,["Pages/Page_55591967.html#PageId_55592010","3.6.3. Sharing Framebuffers to Conserve VRAM (Important)",
"3.6.3. Sharing Framebuffers to Conserve VRAM (Important)\nWhen rendering to each of the screens, you can conserve VRAM by reusing the same framebuffer first for the right eye, then for the left eye, and finally for the lower screen.\n",0]
,["Pages/Page_55591967.html#PageId_55592011","3.6.4. Display Buffer Placement",
"3.6.4. Display Buffer Placement\nWhen the display buffer is placed in main memory, main memory is accessed regularly because the GPU transfers the content of the display buffer to the LCD every frame.\nWhen the display buffer is placed in VRAM, the content of the display buffer is transferred to the LCD without accessing main memory. Although this puts pressure on VRAM, it is useful in avoiding access conflicts when the cameras or other devices access main memory often.\n Note:\n If competing memory access prevents data from being transferred to the LCD on time, artifacts may be displayed on the LCD. You can also use nn::gx::SetMemAccessPrioMode to configure access priorities for main memory.\nCONFIDENTIAL\n",0]
,["Pages/Page_55592012.html","4. Compiler",
"4. Compiler\nThis chapter provides information about compilers and the optimization of generated code.\n",0]
,["Pages/Page_55592012.html#PageId_55592013","4.1. Coding With Optimization in Mind",
"4.1. Coding With Optimization in Mind\n",0]
,["Pages/Page_55592012.html#PageId_55592014","4.1.1. Using the __restrict Keyword",
"4.1.1. Using the __restrict Keyword\nBy using the __restrict keyword, you can tell the compiler that a pointer or an array of function parameters do not overlap in memory (for floating-point matrix operations, for example). This allows the compiler to perform optimizations that may not be possible when aliases are generated.\nExample:\n MTX44*\nMTX44Multiple(MTX44* pOut, const MTX44* __restrict p1, const MTX44* __restrict p2)\n{\n //･･･\n}\nThe __restrict keyword has no effect if it is used in a template class method, but it is effective when attached to methods that are specialized for float values. The __restrict keyword also works when it is added only to the CPP file (the definition) and not the H file (the header).\n",0]
,["Pages/Page_55592012.html#PageId_55592015","4.1.2. Using the register Modifier",
"4.1.2. Using the register Modifier\nIn a calculation involving floating-point matrices, for example, you can make dependencies between variables explicit by storing the values to be calculated in local variables declared with the register keyword. It then becomes easier to generate code that better accounts for latency by combining independent calculations and so on.\nExample:\n MTX34* MTX34Multiple(MTX34*__restrict pOut, const MTX34*__restrict p1, const MTX34*__restrict p2)\n{\n const f32 (*const a)[4] = p1->m;\n const f32 (*const b)[4] = p2->m;\n register f32 m[3][4];\n // Store the calculated result in register variables\n m[0][0] = a[0][0] * b[0][0] + a[0][1] * b[1][0] + a[0][2] * b[2][0];\n //...\n pOut->m[0][0] = m[0][0];\n //...\n}\n",0]
,["Pages/Page_55592012.html#PageId_55592016","4.1.3. Using Inline Assembly Code",
"4.1.3. Using Inline Assembly Code\nYou must use embedded assembly code to specify VFP instructions directly, but functions written in embedded assembly code cannot be optimized or inlined.\nIf you want the compiler to inline or optimize your code, you must use inline assembly code instead.\nInstructions written using inline assembly code may be expanded to several instructions. The LDM, STM, LDRD, and STRD instructions may be expanded to the LDR and STR instructions.\n Note:\n In ARMCC 4.1 and subsequent compiler versions, VFP instructions can be written using inline assembler code.\n",0]
,["Pages/Page_55592012.html#PageId_55592017","4.1.4. Optimization Warnings",
"4.1.4. Optimization Warnings\nIf RVCT determines that it cannot optimize a function, it outputs information similar to the following and stops optimizing that particular function (after expanding inline statements). This causes the slowest code to be output. You can improve performance by revising your C++ code so that it does not prevent optimization when information such as the following is displayed.\n #1596-D: Could not optimize: External function reference prevents optimization\nPay attention to output related to optimization that begins with “Could not optimize:” or “Might not be able to optimize:”.\n Note:\n The CTR-SDK build system suppresses optimization warnings. To display these warnings, you must remove the --diag_suppress=optimizations option from CCFLAGS_WARNING in $(CTRSDK_ROOT)/build/omake/commondefs.cctype.RVCT.om.\n",0]
,["Pages/Page_55592012.html#PageId_55592018","4.2. Tuning Assembly Levels",
"4.2. Tuning Assembly Levels\n",0]
,["Pages/Page_55592012.html#PageId_55592019","4.2.1. Referencing Disassembly Listings",
"4.2.1. Referencing Disassembly Listings\nYou can use the following steps to disassemble and analyze the code output by the compiler.\nThe CTR-SDK build system outputs a file with the .dasm extension, holding the disassembly results to the same location as the executable image.\nIf --interleave=source_only is added to the DISAS flag in $(CTRSDK_ROOT)/build/omake/commondefs.cctype.RVCT.om, the output file will include C source code.\nTo disassemble a single function, specify its path to armcc.exe as follows.\n armcc.exe --debug -O3 -Otime --cpu MPCore --interleave -S test.cpp\nWhen this is compiled with the -S option, only assembly code is generated. The output file has a .s extension.\nAdding the --interleave option causes the output file to contain C code and a .txt extension.\n",0]
,["Pages/Page_55592012.html#PageId_55592020","4.2.2. Analyzing CPU Stalls",
"4.2.2. Analyzing CPU Stalls\nYou can analyze interlock delays caused by the processor pipeline, such as CPU stalls caused by VFP latency, using the relevant assembler warning messages.\nCompile the source to be analyzed with the -S option specified; the assembly source is output.\n armcc.exe --debug -O3 -Otime --cpu MPCore -S test.cpp\nWarnings related to interlock delays are output when the generated assembly source is assembled with the --diag_warning 1563 option specified.\n armasm --diag_warning 1563 test.s --cpu=MPCore test.s\n",0]
,["Pages/Page_55592012.html#PageId_55592021","4.3. Reducing General Processing",
"4.3. Reducing General Processing\nYou can reduce CPU processing to some extent by paying attention to the following points.\n \tAvoid frequent use of virtual. \tReduce function jumps using inline or some other means.\nCONFIDENTIAL\n",0]
,["Pages/Page_55592022.html","5. Synchronization Objects",
"5. Synchronization Objects\nThis chapter provides information about the class of synchronization objects that are provided with the OS library in the CTR-SDK.\n",0]
,["Pages/Page_55592022.html#PageId_55592023","5.1. Mutex and Critical Section Objects",
"5.1. Mutex and Critical Section Objects\nMutex objects and critical section objects are synchronization objects that are locked and released. These objects can be used for purposes of exclusive control. When such an object is locked by a thread, access to the resource from other threads is prohibited until the locking thread has released its lock.\nAs synchronization objects, mutex and critical section objects work basically the same way. But because of the points noted below, the use of critical section objects is highly recommended.\n \tFaster speeds are realized with critical section objects in situations where large amounts of memory are used and where there are no conflicts in locking the objects. \tThere are limits on the number of mutex objects that can be created, but there are no such limits on critical section objects, and you can make as many of the objects as memory allows.\nHowever, critical section objects do not inherit the thread priority and in some cases may end up with the opposite priority. As a result, you must be careful about exclusive control among threads with different priorities.\n",0]
,["Pages/Page_55592022.html#PageId_55592024","5.2. Event and LightEvent Objects",
"5.2. Event and LightEvent Objects\nLightEvent (nn::os::LightEvent class), which is the provided light version of Event (nn::os::Event class), works the same as Event except for small differences, such as the inability to specify timeouts and the waiting for multiple objects. There are limits on the number of Event objects that you can create, but there are no such limits on LightEvent, and you can make as many of the objects as memory allows.\nIn some cases when using the CTR-SDK libraries, event objects are required. The use of LightEvent is recommended where threads are synchronized in the application.\nIn the code example shown below, the process of waiting by multiple objects is replicated using LightEvent.\n Code 5-1. Example of Using LightEvent to Replicate WaitAny\n nn::os::LightEvent event1, event2;\n{\n event1.Initialize(false);\n event2.Initialize(false);\n}\n// ...\n{\n while(!(event1.TryWait() | event2.TryWait()))\n nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(1));\n}\nLightEvent is initialized as an auto-reset event to wait for any of the LightEvent objects to become the signal state.\n Code 5-2. Example of Using LightEvent to Replicate WaitAll\n nn::os::LightEvent event1, event2;\n{\n event1.Initialize(true);\n event2.Initialize(true);\n}\n// ...\n{\n while(!(event1.TryWait() & event2.TryWait()))\n nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(1));\n // Clear Signal\n event1.ClearSignal();\n event2.ClearSignal();\n}\nThis code example uses a manual reset event. If an auto-reset event had been used, the signal state would have been deleted by the TryWait function, and you would need to implement some measure like storing the bitwise OR in a bool type variable.\n",0]
,["Pages/Page_55592022.html#PageId_55592025","5.3. Semaphore and LightSemaphore Objects",
"5.3. Semaphore and LightSemaphore Objects\nThe relation between Semaphore (nn::os::Semaphore class) and LightSemaphore (nn::os::LightSemaphore class) is the same as that between Event and LightEvent.\nCONFIDENTIAL\n",0]
,["Pages/Page_55592026.html","6. File System",
"6. File System\nThis chapter provides information about the CTR-SDK file system (FS).\nIt summarizes how performance is improved or degraded depending on how the CTR-SDK FS library is used. This chapter is mainly based on measurements taken using CTR-SDK 4.x, but the differences in subsequent versions are minimal.\nFor benchmark results and other detailed data, see Section 8.7. FS Library Performance Details.\n",0]
,["Pages/Page_55592026.html#PageId_55592027","6.1. ROM Archive",
"6.1. ROM Archive\nThis section describes the access rates to ROM archives when using development cards (CTR flash cards) or CARD2.\n",0]
,["Pages/Page_55592026.html#PageId_55592028","6.1.1. Batch Read",
"6.1.1. Batch Read\nDue to CTR flash card and CARD2 characteristics, batch reading of data in the ROM archives is most effective when data sizes are 512 KB or greater. Performance decreases when batch reading in data sizes of 256 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592029","6.1.2. Stream Reads",
"6.1.2. Stream Reads\nAs described in Section 6.1.1. Batch Read, when reading files in the ROM archives, the most effective stream reading is performed with a buffer size of 512 KB or more. Performance decreases when batch reading in a buffer size of 256 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592030","6.2. Save Data",
"6.2. Save Data\nThis section describes the access rates to the backup region measured with development cards (CTR flash cards) or CARD2.\n Note:\n Access speeds for save data improve when using CARD2. Specifically, read operations improve by approximately five times and write operations improve by approximately 30 times.\n",0]
,["Pages/Page_55592026.html#PageId_55592031","6.2.1. Reading",
"6.2.1. Reading\nThere are no large differences for batch reads of save data in the backup region due to the data size.\nFor CARD2, because efficiency is poor when the file is very small, access rates decrease when the size is 128 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592032","6.2.2. Writing",
"6.2.2. Writing\nFor writing save data to the backup region, because efficiency is poor when the file is very small, access rates decrease when the size is 32 KB or less.\nFor CARD2, because efficiency is poor when the file is very small, access rates decrease when the size is 128 KB or less.\nWhen the application supports the auto-save feature, because the time when processing can be performed until the power is disconnected, Nintendo recommends not using save data of a large size.\n",0]
,["Pages/Page_55592026.html#PageId_55592033","6.2.3. Mounting",
"6.2.3. Mounting\nWhen formatting, mounting, and unmounting the backup region, it takes much more processing time to format a backup region when data has been written to it than when the backup region is in the factory state. The time it takes to format the region also depends on the amount of data written.\n",0]
,["Pages/Page_55592026.html#PageId_55592034","6.2.4. Deleting Files",
"6.2.4. Deleting Files\nThe average time required for file deletion is 658.566 ms without automatic redundancy and 1.465 ms with automatic redundancy.\n",0]
,["Pages/Page_55592026.html#PageId_55592035","6.3. Extended Save Data",
"6.3. Extended Save Data\nThis section describes the access speeds for accessing the extended save data region on media having the same specifications and capacity as the SD Card packaged with the CTR system.\n",0]
,["Pages/Page_55592026.html#PageId_55592036","6.3.1. Mounting",
"6.3.1. Mounting\nThe average processing times for mounting, deleting, creating, remounting, and unmounting expanded save data are 40.844 ms (mounting), 105.296 ms (deleting), 2346.382 ms (creating), 38.256 ms (remounting), and 1.842 ms (unmounting), respectively.\n",0]
,["Pages/Page_55592026.html#PageId_55592037","6.3.2. Creating Files",
"6.3.2. Creating Files\nThe average processing time to create a file on expanded save data is 766.200 ms.\n",0]
,["Pages/Page_55592026.html#PageId_55592038","6.3.3. Reading Files",
"6.3.3. Reading Files\nFor batch reading files on expanded save data, because efficiency is poor when the file is very small, performance decreases when the size is 256 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592039","6.3.4. Writing Files",
"6.3.4. Writing Files\nFor writing files to expanded save data, speeds tend to increase as the data size increases.\n",0]
,["Pages/Page_55592026.html#PageId_55592040","6.3.5. Deleting Files",
"6.3.5. Deleting Files\nThe average processing time to delete a file from expanded save data is 196.322 ms.\n",0]
,["Pages/Page_55592026.html#PageId_55592041","6.4. ROM Archives (Downloadable Applications)",
"6.4. ROM Archives (Downloadable Applications)\nThis section describes the access speeds for accessing a downloadable application’s ROM archives on media with the same specifications and capacity as the SD Card bundled with the CTR system.\n",0]
,["Pages/Page_55592026.html#PageId_55592042","6.4.1. Batch Reading",
"6.4.1. Batch Reading\nBatch reading is most effective when data sizes are 512 KB or greater. Performance decreases when batch reading in data sizes of 256 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592043","6.4.2. Stream Reads",
"6.4.2. Stream Reads\nStream reading of files is most effective with a buffer size of 512 KB or more, similar to the characteristics described in Section 6.4.1. Batch Read. Performance decreases when batch reading in a buffer size of 256 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592044","6.5. Save Data (Downloadable Applications)",
"6.5. Save Data (Downloadable Applications)\nThis section describes the access speeds for accessing a downloadable application's backup region on media with the same specifications and capacity as the SD Card bundled with the CTR system.\n",0]
,["Pages/Page_55592026.html#PageId_55592045","6.5.1. Reading",
"6.5.1. Reading\nFor batch read operations of save data written to the backup region of a downloadable application, performance decreases for data of 32 KB or less.\n",0]
,["Pages/Page_55592026.html#PageId_55592046","6.5.2. Writing",
"6.5.2. Writing\nFor writing save data to the backup region of a downloadable application, speeds tend to increase as the data size increases.\n",0]
,["Pages/Page_55592026.html#PageId_55592047","6.5.3. Mounting",
"6.5.3. Mounting\nThe average processing times for formatting, mounting, and unmounting the backup region of a downloadable application are 436.077 ms (formatting without automatic redundancy), 197.499 ms (formatting with automatic redundancy enabled), 24.674 ms (mounting), and 1.338 ms (unmounting), respectively.\n",0]
,["Pages/Page_55592026.html#PageId_55592048","6.5.4. Deleting Files",
"6.5.4. Deleting Files\nThe average processing times to delete a file from the backup region of a downloadable application are 171.093 ms without automatic redundancy and 1.546 ms with automatic redundancy enabled.\n",0]
,["Pages/Page_55592026.html#PageId_55592049","6.6. Flushing File Write Operations (Important)",
"6.6. Flushing File Write Operations (Important)\nWe recommend flushing file write operations after writing the entire file has completed.\nThe time required to write files of the same size may be significantly longer in the case of an implementation in which flushing is performed at the same time as writing compared to an implementation in which flushing is performed after the entire file has been written. The difference in execution time grows larger as the amount of data written per function call becomes smaller, resulting in a difference several times larger depending on the total size.\n If true is specified in the argument flush when using TryWrite, flushing is performed while writing. However, note that if false is specified in flush, flushing is not performed unless explicitly carried out using TryFlush or a similar function.\n Code 6-1. Sample Implementation of Flushing When Writing\n size_t restSize = fileSize;\nwhile (restSize > 0)\n{\n size_t writeSize = (restSize < BUFFER_SIZE) ? restSize : BUFFER_SIZE;\n s32 actualSize = 0;\n fileStream.TryWrite(&actualSize, s_pBuffer, writeSize, true);\n restSize -= writeSize;\n}\nPerformance when writing files may be improved by making the following changes.\n Code 6-2. Sample Change in Implementation\n size_t restSize = fileSize;\nwhile (restSize > 0)\n{\n size_t writeSize = (restSize < BUFFER_SIZE) ? restSize : BUFFER_SIZE;\n s32 actualSize = 0;\n // Do not flush while writing the file by specifying false for the fourth argument\n fileStream.TryWrite(&actualSize, s_pBuffer, writeSize, false);\n restSize -= writeSize;\n}\n// Flush after writing the entire file is complete\nfileStream.TryFlush();\nMaking these changes when writing to an SD Card-equivalent product packaged with the system increases performance by about nine times (improving total write time from 11.717 seconds to 1.290 seconds) when writing two files of size 352 KB and 192 KB, respectively, and using a BUFFER_SIZE of 8 KB.\n Note:\n These performance figures were measured under conditions of normal access priority (PRIORITY_APP_NORMAL).\n",0]
,["Pages/Page_55592026.html#PageId_55592050","6.7. Performance of SD Cards Packaged With the System (Important)",
"6.7. Performance of SD Cards Packaged With the System (Important)\nThe SD cards that are currently packaged with systems for resale are supplied by multiple manufacturers. This means that the systems on the market include media with different levels of performance.\nThe media from manufacturers that were added after the launch of the Nintendo 3DS have better performance than the media packaged at launch when writing to random locations and when writing small files. On the other hand, performance is slightly worse when writing files that are several megabytes in size directly to the SD Card. There is no significant difference in performance in terms of reading data.\nUse media packaged with development hardware when debugging applications. Although the same media as packaged with development hardware in the past will continue to be used, media having different performance from conventional media may be included with development hardware in the future. If this happens, use the media packaged with the development hardware, rather than the old media.\nCONFIDENTIAL\n",0]
,["Pages/Page_55592051.html","7. Processing Handled by the System Core",
"7. Processing Handled by the System Core\nThe CPU in a 3DS system has two cores. The application core can be used freely by applications, while the system core is used by the system to handle device processing. Processing for each device is assigned a priority level, so an increase in one device's processing load may affect other devices. The handling of processes in the system core varies depending on which version of the SDK you are using.\nThe following list of processes handled by the system core are directly related to the application.\n \tCameras \tGraphics \tHID \tWireless Communication\n Note:\n In addition to these processes, some API functions called by an application (such as FS library API functions) are also processed in the system core.\nGeneral information about the processing loads for these devices follows.\n",0]
,["Pages/Page_55592051.html#Anchor_55592051_カメラ","Cameras",
"Cameras\nThese processes are involved in obtaining camera images. There is almost no load when the cameras are not being used. The camera’s processing load varies with settings such as the frame rate. Y2R processing is included in this item.\n",0]
,["Pages/Page_55592051.html#Anchor_55592051_グラフィックス","Graphics",
"Graphics\nThe system core synchronizes the GPU and command lists when command lists are executed. Because this involves an interrupt and execution of the next request for every command request, the associated processing load is based primarily on the number of command requests.\n",0]
,["Pages/Page_55592051.html#Anchor_55592051_HIDデバイス","HID",
"HID\nThe HID (human interface device) executes processing for key input, the touch panel, the accelerometer, and the gyro sensor.\nAlthough the accelerometer and gyro sensor create no processing load when they are not used (when they are shut down), it is not possible to shut down the processing of the other devices. If the accelerometer and gyro sensor are used, the accelerometer puts a larger load on the system core than the gyro sensor, but the overall processing load of the other devices is larger than that of the accelerometer (other devices > accelerometer > gyro sensor). If the debug controller is connected to development hardware, the processing load increases even more.\n",0]
,["Pages/Page_55592051.html#Anchor_55592051_無線通信","Wireless Communication",
"Wireless Communication\nThese processes are related to wireless communications. Wireless communication has low priority in the system core, and generally does not affect other processing. However, because it is highly susceptible to the effects of other processing, the smaller the processing load by higher-priority applications, the more stable wireless communication becomes.\n",0]
,["Pages/Page_55592051.html#PageId_55592058","7.1. Processing in the System Core",
"7.1. Processing in the System Core\nSome system core processing resources are available to the application. The application can operate essential processes in the system core, but a limit is set on the application’s use of the system core. This limit helps to guarantee future operation of system and application processes in the system core.\nThe maximum amount of system core CPU time that can be assigned by the nn::os::SetApplicationCpuTimeLimit function to application processes is 30%.\n Note:\n To run sounds in the system core using NW4C, the associated processing must be carried out within the CPU time allocated to the application. Noise might be introduced during sound playback if other processes are also being handled by the system core at the same time.\n In extended applications, even if the same ratio of CPU time is allocated to the application, more processes can be handled due to the increased CPU clock rate.\nThe following processes operate in the system core, and are listed in order of priority from highest to lowest.\n \tCameras \tGraphics \tHID (including Circle Pad Pro) \tWireless Communication\n",0]
,["Pages/Page_55592051.html#PageId_55592059","7.1.1. Changing the Priority of Graphics Processing",
"7.1.1. Changing the Priority of Graphics Processing\n You can change the priority of graphics processing in the system core from high to low using the nngxSetInternalDriverPrioMode function. When graphics processing is set to low priority, the order of priority of the processes carried out in the system core changes as follows.\n \tCameras \tHID (including Circle Pad Pro) \tGraphics \tWireless Communication\nChanging the priority of graphics makes no significant difference in the performance of the application.\n",0]
,["Pages/Page_55592051.html#PageId_55592060","7.1.2. Cautions When Using the System Core",
"7.1.2. Cautions When Using the System Core\n Some CPU time of the system core is allocated to applications using nn::os::SetApplicationCpuTimeLimit to allow the creation of threads that run in the system core.\nAlthough running some processes in the system core allows you to increase the processes being executed by an application, some processes are best suited for execution in the system core and some are not because system core scheduling allocates CPU time in 2-ms intervals.\n Note:\n System processes cannot be run during CPU time allocated to an application even if the application is not using the system core at the time. The execution time of system processes, such as wireless communication, decreases by the amount of system core allocated to the application.\n The CPU time allocated to an application is not synced with picture frames.\n Note:\n For information about system core scheduling, see the System Programming Guide included in the CTR-SDK documentation.\n",0]
,["Pages/Page_55592051.html#Anchor_55592060_システムコアで行うことに向いている処理","Processing Suited for Execution in the System Core",
"Processing Suited for Execution in the System Core\nThe system core CPU time allocated to applications is fixed. This CPU time is best suited for processes such as the following that occur regularly and are completely CPU-bound.\n \tReading and writing buffers.\nExamples include processes that write data (such as movie data or raw sound data) loaded in memory to a work buffer, and processes that copy data between a send/receive buffer used in wireless communication and a work buffer.\n \tGenerating GPU commands. \tAI processes, computational processes, and so on.\nEven in cases like these, processing may take more time than execution in the application core due to the graphics or sound interrupts and the fact that time is allocated in specific intervals. We recommend that you execute processes that allow a certain degree of delay.\n",0]
,["Pages/Page_55592051.html#Anchor_55592060_システムコアで行うことに向いていない処理","Processes Not Suited for Execution in the System Core",
"Processes Not Suited for Execution in the System Core\nProcesses that are not completely CPU-bound and processes that only reoccur a few times during the course of executing an application are unsuited for execution in the system core (except when you want to emphasize the process).\nMake sure to pay particular attention in cases where processes use devices other than graphics and sound-related devices. If an application calls an API function that uses such devices, that process starts when transferred to the system after the system core CPU time allocated to the application passes. In other words, even if the process completes in a short amount of CPU time, the result cannot be obtained by the application until the next interval of CPU time allocated to the application occurs. If a process does not complete within the time allocated to the application, thread processes are completely blocked during the allocated CPU time.\nEven though you can reduce the possibility of blocking during wireless communication by splitting send and receive processing into different threads, processes are blocked for a full 2 ms if the number of packets being sent and received is greater than the number that can be handled during the time system processing can execute.\n",0]
,["Pages/Page_55592051.html#PageId_55592063","7.1.3. Resetting the Allocated Time (Important)",
"7.1.3. Resetting the Allocated Time (Important)\n You can reset the system core CPU time allocated to an application by calling the nn::os::SetApplicationCpuTimeLimit function again. Resetting the allocation is useful in cases where the amount of processing by the application changes for different scenes, such as with single player and multiplayer modes. Note however, that once set there is no returning to zero, and at least 5% (or approximately 0.1 ms) of an interval will definitely be allocated to the application. Changing the allocation frequently is very risky. We recommend changing the allocation only for high level functionality such as “single player” and “multiplayer.”\nAllocating system core CPU to an application has a significant effect on the wireless communication being carried out by the system. We recommend that you do not allocate system core CPU time to applications during multiplayer modes that use wireless communication. To prioritize multiplayer mode, either do not allow applications to use the system core in the first place or make sure that the amount of time allocated is short and has low real-time requirements.\nFor your reference, we carried out a comparison of communication status while varying the allocation time for an application (equivalent to the sync/simple demo in the CTR-Pia library) that carries out only communication processes. Results showed that communications were very stable when no system core CPU time was allocated in the first place. On the other hand, packet loss apparently proportional to the load on the system core even when threads running in the system core were not created occurred when up to 30% (approximately 0.6 ms) of a CPU time interval was allocated.\nComparison was carried out under the following conditions.\n Table 7-1. Conditions Used to Compare Communication Status Both Using and Not Using the System Core\n \t Item\n \t Condition\n \t Protocol\n \t Local UDS communication\n \t Network\n \t Single network consisting of four 3DS systems\n \t Number of packets\n \t 60 packets per second per unit (all set to Broadcast)\n \t Wireless signal\n \t Extremely good\nCONFIDENTIAL\n",0]
,["Pages/Page_55592064.html","8. Appendix",
"8. Appendix\n",0]
,["Pages/Page_55592064.html#PageId_55592065","8.1. Profiling CPU and GPU Operations",
"8.1. Profiling CPU and GPU Operations\nIf you run the GPU during CPU processing, the CPU receives interrupts from the GPU and competes with the GPU to access main memory. These delays make it impossible to accurately measure execution time.\nTo accurately measure the execution time of the CPU and GPU, you must wait for an operation on the CPU to finish completely before running an operation on the GPU.\nThe following procedure provides a specific example of how to do this.\n \t Call nngxClearCmdlist to clear the command list. \t Call nngxStopCmdlist to stop execution of the GPU. \t Call nngxBindCmdlist to bind the command list. \tStart measuring the CPU time. \tIssue commands from the CPU. \tStop measuring the CPU time. \t Call nngxSplitCmdlist to issue a command request (render command request). \tStart measuring the GPU time. \t Call nngxRunCmdlist to issue command lists from the GPU. \t Call nngxWaitCmdlistDone to wait for the GPU to finish its operations. \tStop measuring the GPU time.\n Related Functions:\n nngxClearCmdlist, nngxStopCmdlist, nngxBindCmdlist, nngxRunCmdlist, nngxWaitCmdlistDone\n",0]
,["Pages/Page_55592064.html#PageId_55592066","8.2. Reducing the Processing Load to Display Model Data (Important)",
"8.2. Reducing the Processing Load to Display Model Data (Important)\nThe processing load to display a model is split between the CPU, vertex shader (GPU), and fragment shader (GPU). To reduce the processing load, you must determine which process is the bottleneck when you create model data. Also, if you place texture data and vertex data in main memory, note that access conflicts may cause the CPU performance to decline.\nThe following figure shows a procedure for reducing the processing load to display model data.\n Figure 8-1. Reducing the Processing Load to Display Model Data\n",0]
,["Pages/Page_55592064.html#PageId_55592067","8.3. Hardware Configuration",
"8.3. Hardware Configuration\nThis section describes the configuration of the main hardware related to graphics.\n When an application runs as a standard application on SNAKE, the hardware configuration is identical to CTR.\n Figure 8-2. Hardware Configuration (Standard Application)\n",0]
,["Pages/Page_55592064.html","Hardware Configuration for Extended Applications",
" Hardware Configuration for Extended Applications\n When an application runs as an extended application on SNAKE, the CPU performance and main memory change as illustrated below.\n Figure 8-3. Hardware Configuration (Extended Application)\n",0]
,["Pages/Page_55592064.html#PageId_55592068","8.3.1. CPU",
"8.3.1. CPU\nThe CPU generates graphics commands and conveys instructions to the GPU. One of the CPU cores is used exclusively by applications as the application core, while the remaining core is used as the system core. The CPU is connected to the GPU, main memory, and other modules via the main bus.\n When an application runs as an extended application on SNAKE, the CPU clock rate increases to 804 MHz and the 2 MB L2 cache shared by all CPU cores becomes available.\n",0]
,["Pages/Page_55592064.html#PageId_55592069","8.3.2. Main Memory",
"8.3.2. Main Memory\nMany kinds of data can be placed in main memory. A maximum of 64 MB of main memory can be used from the application. This main memory is connected to the main bus.\n When an application runs as an extended application on SNAKE, up to 124 MB of memory is available for use by the application.\n",0]
,["Pages/Page_55592064.html#PageId_55592070","8.3.3. DMA",
"8.3.3. DMA\nThis is used by command request DMA transfer commands. Its uses include the transfer of texture and vertex data from main memory to VRAM.\n",0]
,["Pages/Page_55592064.html#PageId_55592071","8.3.4. Main Bus",
"8.3.4. Main Bus\nThis bus connects the CPU, the GPU, main memory and other devices. Many kinds of data are exchanged primarily via this bus. To adjust the priority of modules that use the main bus, use the nngxSetMemAccessPrioMode function.\n",0]
,["Pages/Page_55592064.html#PageId_55592072","8.3.5. GPU",
"8.3.5. GPU\nThe GPU comprises the P3D, PPF, PDC, VRAM A, and VRAM B modules, and the internal bus that connects those modules.\n Table 8-1. The GPU Modules\n \t Module\n \t Description\n \t P3D\n \t This module references the 3D commands accumulated in the command buffer and performs the actual rendering.\n \t PPF\n \t This module performs block-to-linear conversion based on command request post-transfer commands and transfers data.\n \t PSC\n \t This module performs memory fill commands (clearing) for command requests.\n \t PDC\n \t This module transfers the content of the display buffer to the LCD.\n \t VRAM A/VRAM B\n \t Memory where vertices and textures are placed. VRAMA and VRAMB can operate separately.\n \t GPU internal bus\n \t The bus that connects the modules in the GPU. It is connected to the main bus.\n Note:\n The GPU internal bus is around twice the bandwidth of the main bus. For data that is frequently accessed in the GPU, you can boost the access speed by placing the data in VRAM, which also lifts the load off the main bus.\n When data is exchanged between the GPU internal bus and the main bus, it is the main bus that is the rate-limiting factor.\n",0]
,["Pages/Page_55592064.html#PageId_55592073","8.3.6. LCD",
"8.3.6. LCD\nThis refers to the two 3DS LCD screens. They are connected to the PDC module within the GPU. When images are being displayed, the PDC gets one line of data from the display buffer for each scan line on the LCD. When the display buffer is located in main memory, the fetching of data places a periodic load on the main bus.\n",0]
,["Pages/Page_55592064.html#PageId_55592074","8.4. GPU Profiling Feature",
"8.4. GPU Profiling Feature\nYou can use the GPU profiling feature to measure processing in each hardware module in the GPU. The following table lists what information can be obtained using this feature.\n Table 8-2. Information Obtainable With the Profiling Feature\n \t Information\n \t Description\n \t Busy clock\n \t You can get the number of busy clock cycles that occur as a result of the vertex shader and fragment lighting in the various GPU modules.\n \t Shader execution clock\n \t You can get the number of execution clock cycles and stall clock cycles for each vertex shader processor.\n \t Number of vertices entered in vertex cache\n \t You can get the number of vertices entered into post vertex cache. By taking the difference from the number of vertices actually used for rendering, you can determine the effective number of vertices in post vertex cache.\n \t Number of input/output polygons\n \t You can get the number of polygons input for triangle setup and the number of polygons output after clipping.\n \t Number of input fragments\n \t You can get the number of fragments input to the per-fragment operation module.\n \t Number of accesses to memory\n \t You can get the number of times memory is accessed by each GPU hardware module.\nThe busy-clock obtained for each hardware module includes not only the busy clock cycles related to the actual processing of data, but also the time that the module is waiting and cannot output data because later modules are busy. It is effective to optimize not only modules that have particularly high busy-clock values, but also the final-stage module among modules with high busy values.\n Note:\n For more information on the profiling feature, see the CTR Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55592064.html#PageId_55592075","8.5. Notes for Using the DMPGL",
"8.5. Notes for Using the DMPGL\nThe following sections provide notes about performance when using the DMPGL.\n",0]
,["Pages/Page_55592064.html#PageId_55592076","8.5.1. Maintaining Internal State Consistency",
"8.5.1. Maintaining Internal State Consistency\nThe internal state refers to local data that is saved by the DMPGL driver; it could also be called a mirror of the hardware settings. Because DMPGL function calls update the internal state and commands configure the hardware settings, the DMPGL driver’s internal state may become inconsistent with the hardware settings when commands are issued directly or when command caches are used.\nIf there is a discrepancy between the internal state and hardware settings, you can preserve consistency by forcing the hardware settings to be validated (by issuing complete command packets). Validation actually occurs after nngxUpdateState(NN_GX_STATE_ALL) is run.\nThis issues commands only for hardware settings that have been changed by the command cache or by commands that were issued directly, rather than for all settings, allowing you to trim the process of issuing commands. However, applications must carefully keep track of which states correspond to and which states are dependent on the commands that have been used.\nWhen nngxSetCommandGenerationMode(NN_GX_CMDGEN_MODE_UNCONDITIONAL) is called, commands are issued regardless of the comparison results for the internal state. Only the following settings are affected by this mode.\n \tUniform settings for the reserved fragment shader. \tInteger uniform settings for the vertex shader. \tLUT data settings. \t DMPGL functions associated with NN_GX_STATE_OTHERS.\n Related Functions:\n nngxUpdateState, nngxValidateState, glDrawElements, glDrawArrays, nngxSetCommandGenerationMode\n",0]
,["Pages/Page_55592064.html#PageId_55592077","8.5.2. Removing the Use of glGetUniformLocation (Important)",
"8.5.2. Removing the Use of glGetUniformLocation (Important)\nOne way to configure a shader uniform using DMPGL involves using the glGetUniformLocation function to get its location. Because it performs processor-intensive operations like string comparison, heavy use of glGetUniformLocation is not recommended.\nTo get Location values for the fixed fragment shader, use the constants defined for that purpose. The locations of each uniform of a program object are guaranteed not to change until the program object is either destroyed by the glDeleteProgram function or relinked by the glLinkProgram function.\n Note:\n Macros for the Location values are defined in $CTR_SDK/include/nn/gx/CTR/gx_UniformLocationForFragmentShader.h.\n Related Functions:\n glGetUniformLocation, glDeleteProgram, glLinkProgram\n",0]
,["Pages/Page_55592064.html#PageId_55592078","8.5.3. Cost of Switching Programs With glUseProgram",
"8.5.3. Cost of Switching Programs With glUseProgram\n When glUseProgram is called, a shader binary must be loaded during validation if the new program object and the old program object link to different shader binaries.\nIt is not recommended to render by frequently switching program objects that link to separate shader binaries. As an effective alternative, consider using a conditional branch instruction to allow your vertex shader binaries to be shared by multiple program objects, or adjust the render order to minimize the number of calls to glUseProgram.\n Related Functions:\n glUseProgram\n",0]
,["Pages/Page_55592064.html#PageId_55592079","8.5.4. Cost of Calling glUseProgram(0)",
"8.5.4. Cost of Calling glUseProgram(0)\n Calling glUseProgram(0) causes all validation flags to be set the next time glUseProgram is called, making shared shader binaries ineffective. DMPGL is designed to monitor differential updates, so there is no need to call glUseProgram(0) explicitly.\n Related Functions:\n glUseProgram\n",0]
,["Pages/Page_55592064.html#PageId_55592080","8.5.5. Notes About the transpose Parameter of glUniformMatrix",
"8.5.5. Notes About the transpose Parameter of glUniformMatrix\nMatrices are treated as column-major in OpenGL ES. The transpose parameter of the glUniformMatrix functions, which indicates whether a matrix is transposed before it is loaded, can only be set to GL_FALSE. In contrast, DMPGL treats matrices as row-major within the graphics driver, and allows GL_TRUE to be specified as the transpose parameter of the glUniformMatrix functions.\nDue to internal differences between DMPGL and OpenGL ES, matrices are implicitly transposed if GL_FALSE is specified in the transpose parameter of the glUniformMatrix functions.\nMatrices generated using the MATH library of the CTR-SDK, on the other hand, are row-major just like DMPGL drivers. By specifying GL_TRUE in the transpose parameter of the glUniformMatrix functions, you can set matrices without transposing them.\n Related Functions:\n glUniformMatrix\n",0]
,["Pages/Page_55592064.html#PageId_55592081","8.5.6. Using Textures in the Native PICA Format",
"8.5.6. Using Textures in the Native PICA Format\nThe current CTR-SDK specifications support textures in standard OpenGL format. Using the standard format requires that the textures be converted to the native format when glTexImage2D is called. (This conversion is done automatically.) This conversion can be omitted by maintaining textures in native format instead.\n Related Functions:\n glTexImage2D\n",0]
,["Pages/Page_55592064.html#PageId_55592082","8.5.7. Setting Uniforms for Vertex Shaders",
"8.5.7. Setting Uniforms for Vertex Shaders\nOverwriting the uniforms in a vertex shader is more efficient if the registers being overwritten are contiguous.\n \tCase 1: Overwrite all registers ([c0...c15]). \tCase 2: Overwrite non-contiguous blocks of registers ([c0...c3], [c8...c11], [c16...c19], [c24...c27]).\nIn both cases, the number of registers being overwritten is the same, but Case 1 is more efficient because it allows transfer commands to be consolidated.\n Related Functions:\n glUniform\n",0]
,["Pages/Page_55592064.html#PageId_55592083","8.5.8. Updating Buffers",
"8.5.8. Updating Buffers\nThe glBufferData and glTexImage2D functions, which rewrite vertex and texture data, access data in the main memory. Accessing this data requires the CPU to apply the changes to the GPU. Because this process puts a relatively large overhead on the system, the overall processor load increases proportionally to the number of calls to such functions.\nWe recommend that you call these functions in advance when the application is being initialized, load vertex buffer and texture data early, and avoid loading data during per-frame operations whenever possible.\n The same type of overhead is involved in using the glBufferSubData function to partially update vertex data. To reduce this overhead, you might consider gathering all required data for a partial update into a single chunk and processing it all with a single call to glBufferSubData.\n Related Functions:\n glBufferData, glTexImage2D, glCompressedTexImage2D, glTexImage1D, glBufferSubData, glTexSubImage1D\n",0]
,["Pages/Page_55592064.html#PageId_55592084","8.5.9. Validation",
"8.5.9. Validation\nWith DMPGL, any configuration changes made by calls to the GL API are applied to the hardware. An operation called validation is performed on the command buffer that actually writes the commands. Validation occurs when the following functions are called.\n \t glDrawArray \t glDrawElements \t nngxValidateState\nEach setting is divided into several categories, and a state update flag is set for each category that is updated. During validation, these flags are used to determine which categories to update. Updates are applied one category at a time. The processing load for glDrawArray and glDrawElements increases with the number of categories to update.\n If you use glUseProgram to switch programs, state update flags are only set for categories that differ from the previous program. Details about the functions for each category are shown below. If functions like glUniform and glEnable are used to change the parameters of a particular category, the update flag of that category is set.\n Table 8-3. Categories Updated by Validation\n \t Category\n \t Functions Used\n \t Framebuffers\n \t \t glBindFramebuffer \t glBindRenderbuffer \t glDeleteFramebuffers \t glDeleteRenderbuffers \t glFramebufferRenderbuffer \t glFramebufferTexture2D \t glRenderbufferStorage \t glReadPixels \t glClear\n \t Vertex buffers\n \t \t glBindBuffer \t glBufferData \t glBufferSubData \t glDeleteBuffers\n \t Triangles\n \t \t glEnable \t glDisable \t glUseProgram \t glDepthRangef \t glPolygonOffset\n \t Lighting LUTs\n \t \t glUseProgram \t glUniform*\n dmp_LightEnv.lutEnabledSP\n dmp_LightEnv.lutEnabledD0\n dmp_LightEnv.lutEnabledD1\n dmp_LightEnv.fresnelSelector\n dmp_LightEnv.lutEnabledRefl\n dmp_FragmentMaterial.sampler{D0,D1,SP,FR,RB,RG,RR}\n dmp_FragmentLighting.enabled \t glUniformsDMP \t glRestoreProgramsDMP\n \t Fog LUTs\n \t \t glUseProgram \t glUniform*\n dmp_Fog.mode (GL_FOG or GL_GAS_DMP)\n dmp_Fog.sampler \t glUniformsDMP \t glRestoreProgramsDMP\n \t Procedural texture LUTs\n \t \t glUseProgram \t glUniform*\n dmp_Texture[3].ptSampler{Rgb,Alpha,Noise,R,G,B,A}\n dmp_Texture[3].ptNoiseEnable\n dmp_Texture[3].ptAlphaSeparate\n dmp_Texture[3].samplerType \t glUniformsDMP \t glRestoreProgramsDMP\n \t Vertex arrays\n \t \t glBindBuffer \t glEnableVertexAttribArray \t glDisableVertexAttribArray \t glVertexAttribPointer\n \t Current vertices\n \t \t glBindBuffer\n \t Framebuffer access\n \t \t glEnable \t glDisable \t glDepthFunc \t glEarlyDepthFuncDMP \t glColorMask \t glDepthMask \t glStencilMask \t glUseProgram \t glUniform*\n dmp_FragOperation.mode \t glUniformsDMP \t glRestoreProgramsDMP\n \t Scissor/viewport\n \t \t glEnable \t glDisable \t glScissor \t glViewport\n \t Texture 0\n \t \t glUseProgram \t glUniform*\n dmp_Texture[0].samplerType \t glUniformsDMP \t glRestoreProgramsDMP \t glBindTexture \t glDeleteTextures \t glCompressedTexImage2D \t glCopyTexImage2D \t glCopyTexSubImage2D \t glTexImage2D \t glTexParameteriv \t glTexParameterfv\n \t Texture 1\n \t \tSame as Texture 0 \t glUniform*\n dmp_Texture[1].samplerType\n \t Texture 2\n \t \tSame as Texture 0 \t glUniform*\n dmp_Texture[2].samplerType\n \t Texture 3\n \t \t glUseProgram \t glRestoreProgramsDMP \t glBindTexture \t glDeleteTextures \t glCompressedTexImage2D \t glCopyTexImage2D \t glCopyTexSubImage2D \t glTexImage2D \t glTexParameteriv \t glTexParameterfv\n \t Texture LUTs\n \t \t glBindTexture \t glDeleteTextures \t glTexImage1D \t glTexSubImage1D\n \t Program\n \t \t glUseProgram\n \t Shader uniforms\n \t \t glUseProgram \t glUniform*\nVertex shaders and uniforms for geometry shaders \t glUniformsDMP\n \t Rasterization\n \t \t glDrawArrays \t glDrawElements \t glUniform*\nUniforms for reserved fragment shaders \t glUniformsDMP \t glRestoreProgramsDMP\n \t Shader binaries\n \t \t glUseProgram\n \t Vertex shader binaries\n \t \t glUseProgram\n \t Geometry shader binaries\n \t \t glUseProgram\n \t Geometry shader attachments\n \t \t glUseProgram\n \t Geometry shader detachments\n \t \t glUseProgram\n \t Gas LUTs\n \t \t glUseProgram \t glUniform*\n dmp_Fog.mode (GL_GAS_DMP)\n dmp_Gas.sampler{TR, TG, TB} \t glUniformsDMP\n",0]
,["Pages/Page_55592064.html#PageId_55592085","8.5.10. Functions That Issue Command Requests",
"8.5.10. Functions That Issue Command Requests\n The GPU begins executing commands when nngxRunCmdlist is called. Rendering instructions are processed one command request—accumulated in the command list—at a time.\nThe following functions issue command requests to a command list.\n Table 8-4. Functions That Issue Command Requests\n \t Function\n \t Condition\n \t Command Request Added\n \t nngxSplitDrawCmdlist\n \t Always\n \t Render command request\n \t nngxTransferRenderImage\n \t Always\n \t Render command request\n Post transfer command request\n \t glClear\n \t Always\n \t Render command request\n Memory fill command request\n \t glBufferData\n \t When NN_GX_MEM_VRAMA or NN_GX_MEM_VRAMB is specified as the first argument\n \t DMA transfer command request\n \t glBufferSubData\n \t When glBufferData meets the condition above\n \t DMA transfer command request\n \t glTexImage2D\n \t When NN_GX_MEM_VRAMA or NN_GX_MEM_VRAMB is specified as the first argument\n \t DMA transfer command request\n \t glCopyTexImage2D\n \t Always\n \t Render command request\n Copy texture command request\n \t glCopyTexSubImage2D\n \t Always\n \t Render command request\n Copy texture command request\n \t glRestoreTextureCollectionsDMP\n \t When conditions are met by the functions that generate commands to be restored (glBufferData, glBufferSubData, glTexImage2D).\n \t DMA transfer command request\n \t glRestoreVertexStateCollectionsDMP\n \t glDrawArrays\n \t When the reserved uniform dmp_Gas.autoAcc is GL_TRUE and the function is called for the first time after the reserved uniform dmp_FragOperation.mode has its value changed from GL_FRAGOP_MODE_GAS_ACC_DMP.\n \t Render command request\n \t glDrawElements\n Note 1: Not added when nngxSplitDrawCmdlist is called in advance.\nNote 2: One added per color and depth (stencil) buffer. Two are added when both are specified.\n",0]
,["Pages/Page_55592064.html#PageId_55592086","8.6. Functions That Cause System Blocking",
"8.6. Functions That Cause System Blocking\nIf you measure performance using the CPU profiler, a large proportion of the measurements may be made while the system is blocking (SYSTEM_WAITING). When the system is blocking, it indicates that profiling is being performed on multiple threads, and none of those threads are active. This is because a system call from a particular thread is blocking, but the profiler is not able to determine which thread is blocking.\nYou can use the CPU profiler’s “select thread” and “reverse call tree” features to trace the process calculated as system blocking.\nIf the profiler is tracing the active thread, it is calculated as the system blocking because it cannot identify the last active thread or function. You can eliminate this by using the select-thread feature to limit the threads that are profiled. If you then find a function that was being calculated as system blocking, you can use the reverse call tree to find out where in the application that function is called and then trace it.\nMost CTR-SDK functions have the latent potential to cause the system to block. If possible, Nintendo recommends using the CPU profiler to check performance at crucial points in your implementation.\n",0]
,["Pages/Page_55592064.html#PageId_55592087","8.7. FS Library Performance Details",
"8.7. FS Library Performance Details\nThis section describes the reference data about the access rates to each archive, when the CTR-SDK FS library is used. Specifically, measurement results for file system performance for the ROM archives, save data (backup region), and extended save data (SD Card) are described when the FS library from CTR-SDK 3.x series or later is used. For ROM archives and save data, measurements were made presuming the use of card applications (Nintendo 3DS Game Cards) or downloadable applications (SD Card).\nAlthough the development card speed can be set to both “Fast” and “Slow” for card-based software application ROM archives, almost no devices on the market have a “Slow” setting performance. However, ROM archive performance worsens through repeated access. For example, when reading a 512 KB file, performance like the “Slow” setting only happens momentarily (about one or two internal command cycles) at first, but as the situation worsens “Slow”-like performance gradually becomes more common.\nBecause performance of 3DS Game Cards varies depending on the memory chips inside, use the data provided here as a source of reference. Also note that the performance of CARD2 varies slightly depending on the size of the save region.\nTo measure the performance of SD Cards, we use media that have been formatted with a formatter that complies with the SD File System Specification. Depending on the media capacity and the media maker, the performance may differ even for the same class of media. Because performance of SD cards varies depending on the media and the actual unit, use the data provided here as a source of reference.\nAlso note that the data for CTR-SDK 3.x series and later versions is treated the same because the results were almost identical.\nThe following environment was used for the measurements.\n \tRelease build \tWait simulation function OFF \tDevelopment card speed setting “Fast” \tNormal access priority (PRIORITY_APP_NORMAL)\n Note:\n If you use this feature to access the file system with real-time priority, design your performance based on the data in the documentation bundled with the CTR-SDK, rather than by relying on actual measurements.\n For more information about access priority,see the CTR-SDK documentation.\n Note:\n The performance of ROM archives under emulation in the PARTNER-CTR Debugger has been tuned to nearly match the listed figures (performance of development cards).\n The performance of retail cards is almost the same as the performance of development cards.\n",0]
,["Pages/Page_55592064.html#PageId_55592088","8.7.1. ROM Archive",
"8.7.1. ROM Archive\nThe benchmark data presented in this section is reference data that resulted from measuring access to ROM archives on development cards (CTR flash cards) and CARD2. Because the benchmark data was collected using a sample device with its sole purpose being measurement testing, there is no guarantee that processing will complete within the times presented.\n",0]
,["Pages/Page_55592064.html#PageId_55592089","8.7.1.1. Batch Read",
"8.7.1.1. Batch Read\nThe figures below show the time required by the TryRead function to batch-read data of the specified size 50 times. In short, TryInitialize → TryRead → Finalize was performed 50 times.\nTo prevent cached data from skewing the results, we attempted to only read unique files whenever possible. The blue line is the CTR flash card and the red line is CARD2.\n Figure 8-4. Transfer Rates (Batch Read)\nThe tables below show the time taken by operations other than those of the TryRead function.\n Table 8-5. Time Taken for Processes Other Than the TryRead Function\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t 0.045 ms\n \t 0.021 ms\n \t 0.164 ms\n \t Finalize\n \t 0.006 ms\n \t 0.002 ms\n \t 0.024 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592090","8.7.1.2. Stream Reads",
"8.7.1.2. Stream Reads\nThe figures below show the time required by the TryRead function to read files having the same size 50 times while varying the buffer size. A test unit measured the time required to execute the TryRead function.\nTo prevent cached data from skewing the results, we attempted to read only unique 8-MB files not stored in the cache. The blue line is the CTR flash card and the red line is CARD2.\n Figure 8-5. Transfer Rates (Stream Read)\n",0]
,["Pages/Page_55592064.html#PageId_55592091","8.7.2. Save Data",
"8.7.2. Save Data\nThe benchmark data presented in this section is reference data that resulted from measuring access to the backup region on development cards (CTR flash cards) and CARD2. Because the benchmark data was collected using a sample device with its sole purpose being measurement testing, there is no guarantee that processing will complete within the times presented.\n",0]
,["Pages/Page_55592064.html#PageId_55592092","8.7.2.1. Read",
"8.7.2.1. Read\nThe figures below show the results for batch-reading save data, of the specified size written on a development card or CARD2 that has 512 KB of backup memory, 50 times. In short, TryInitialize → TryRead → Finalize was performed 50 times. The red line shows results obtained with no automatic redundancy, the blue line shows results when automatic redundancy was enabled.\nThe backup memory set for CARD2 was 256 MB. CARD2 data is shown in a separate figure, and the data for the CTR flash card are shown as dashed lines for comparison.\n Figure 8-6. Save Data Read Rates (Average Values)\n Figure 8-7. Comparison of CARD2 and Save Data Read Rates (Average Values)\nTo minimize the impact of caching, the measurement program changed the number of files used based on the capacity. When reading files at the 4-KB mark or less, all the files read were different, but for larger sizes, there was the possibility that caching would have an effect. Note that even when using automatic redundancy, there was little effect when caching files having a size of 16 KB or more because the effect of caching changes depends on factors such as whether a file is read immediately after writing or whether the same file is being read again.\nThe tables below show the time taken by operations other than those of the TryRead function.\n Table 8-6. Time Required for Operations Other Than the TryRead Function (Save Data)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No automatic redundancy\n \t 1.276 ms\n \t 0.708 ms\n \t 7.194 ms\n \t Automatic redundancy enabled\n \t 1.104 ms\n \t 0.715 ms\n \t 7.031 ms\n \t Finalize\n \t No automatic redundancy\n \t 0.480 ms\n \t 0.345 ms\n \t 1.910 ms\n \t Automatic redundancy enabled\n \t 0.395 ms\n \t 0.277 ms\n \t 2.034 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592093","8.7.2.2. Writing",
"8.7.2.2. Writing\nThis section shows the results measured on a test unit for the time required to execute the TryWrite function and the TryFlush function’s operations when writing save data of the specified size 50 times. In short, TryInitialize → TryWrite → TryFlush → Finalize was performed 50 times. Benchmarks were recorded with two types of save data: data saved with and without automatic redundancy enabled. When automatic redundancy was enabled, the measurement also included the operations of the CommitSaveData function.\nThe backup memory set for CARD2 was 256 MB.\n",0]
,["Pages/Page_55592064.html#Anchor_55592093_二重化なし","No Automatic Redundancy",
"No Automatic Redundancy\nThe figures below show the benchmark results for a save data region with no automatic redundancy. The green line shows results obtained when a new file was created (“No file” below), and the blue line shows results when the data overwrote a file of the same size (“Overwrite same size” below).\nCARD2 data is shown in a separate figure, and the data for the CTR flash card are shown as dashed lines for comparison.\n Figure 8-8. Save Data Write Rates (No Automatic Redundancy)\n Figure 8-9. Comparison of CARD2 and Save Data Read Rates (No Automatic Redundancy)\nThere is almost no difference in the time required by the TryWrite function, regardless of whether a file is created or overwritten. The difference in processing speed (of approximately 350 ms) is due to the time taken by the TryFlush function.\nThe tables below show the time taken by the TryInitialize function and the Finalize function under the different measurement conditions.\n Table 8-7. Time Taken by the TryInitialize and Finalize Functions (No Automatic Redundancy)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No file\n \t 620.133 ms\n \t 478.054 ms\n \t 926.344 ms\n \t Overwrite file of the same size\n \t 1.269 ms\n \t 0.709 ms\n \t 3.889 ms\n \t Finalize\n \t No file\n \t 0.489 ms\n \t 0.337 ms\n \t 2.084 ms\n \t Overwrite file of the same size\n \t 0.483 ms\n \t 0.337 ms\n \t 2.203 ms\n The TryInitialize operation takes significantly more time creating a new file compared to when overwriting an existing file.\n",0]
,["Pages/Page_55592064.html#Anchor_55592093_二重化あり","Automatic Redundancy Enabled",
"Automatic Redundancy Enabled\nThe figures below show the benchmark results for a save data region with automatic redundancy enabled. Because automatic redundancy is enabled, the total time includes the operations of the CommitSaveData function in addition to those of the TryWrite and TryFlush functions. The green line shows results obtained when a new file was created (“No file” below), and the blue line shows results when the data overwrote a file of the same size (“Overwrite same size” below).\nCARD2 data is shown in a separate figure, and the data for the CTR flash card are shown as dashed lines for comparison.\n Figure 8-10. Save Data Write Rates (Automatic Redundancy Enabled)\n Figure 8-11. Comparison of CARD2 and Save Data Read Rates (Automatic Redundancy Enabled)\nThe benchmark data shows that whether a file is created or overwritten results in no major difference in processing time, other than the fact that the CommitSaveData operations are around 100 ms faster when data of the same size are being overwritten.\nThe tables below show the time taken by the TryInitialize function and the Finalize function under the different measurement conditions.\n Table 8-8. Time Taken by the TryInitialize and Finalize Functions (Automatic Redundancy Enabled)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No file\n \t 1.267 ms\n \t 0.813 ms\n \t 3.365 ms\n \t Overwrite file of the same size\n \t 1.997 ms\n \t 0.667 ms\n \t 11.511 ms\n \t Finalize\n \t No file\n \t 0.368 ms\n \t 0.259 ms\n \t 1.807 ms\n \t Overwrite file of the same size\n \t 0.384 ms\n \t 0.260 ms\n \t 1.713 ms\n When a file was overwritten as opposed to created, some processes were dramatically slower. Otherwise, the results showed almost no differences under the two measurement conditions.\nThe figures below show the difference in transfer rate with and without automatic redundancy. The red line shows results obtained with no automatic redundancy, and the blue line shows results when automatic redundancy was enabled. In both cases, the measurement condition was “No file.”\n Figure 8-12. Save Data Write Rates (Comparison)\n Figure 8-13. CARD2 and Save Data Write Rates (Comparison)\nIf your application supports the auto-save feature, Nintendo recommends against using a large amount of save data, because there is only a limited amount of time for finishing this process before the system powers off.\n",0]
,["Pages/Page_55592064.html#PageId_55592094","8.7.2.3. Mounting",
"8.7.2.3. Mounting\nThe tables below show the time required, as measured on a test unit, for the following functions: FormatSaveData, MountSaveData, and Unmount. The backup memory size for the development card was 512 KB.\n Table 8-9. Time Required for Operations Related to Mounting Save Data\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t FormatSaveData\n \t No automatic redundancy (factory state)\n \t 2857.726 ms\n \t 2843.064 ms\n \t 2872.938 ms\n \t No automatic redundancy\n \t 8535.636 ms\n \t 8339.167 ms\n \t 8661.502 ms\n \t Automatic redundancy enabled (factory state)\n \t 2476.430 ms\n \t 2462.935 ms\n \t 2495.924 ms\n \t Automatic redundancy enabled\n \t 8186.308 ms\n \t 8015.401 ms\n \t 8278.390 ms\n \t MountSaveData\n \t No automatic redundancy\n \t 26.955 ms\n \t 25.682 ms\n \t 29.150 ms\n \t Automatic redundancy enabled\n \t 30.495 ms\n \t 28.150 ms\n \t 33.798 ms\n \t Unmount\n \t No automatic redundancy\n \t 1.476 ms\n \t 1.081 ms\n \t 3.318 ms\n \t Automatic redundancy enabled\n \t 1.124 ms\n \t 0.873 ms\n \t 2.324 ms\n It takes much more processing time to format a backup region when data has been written to it than when the backup region is in factory condition. The time it takes to format the region also depends on the amount of data written.\n",0]
,["Pages/Page_55592064.html#PageId_55592095","8.7.2.4. Deleting Files",
"8.7.2.4. Deleting Files\nThe tables below show the time required, as measured on a test unit, for the TryDeleteFile function. The backup memory size of the development card was 512 KB.\n Table 8-10. Time Required for the TryDeleteFile Function\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryDeleteFile\n \t No automatic redundancy\n \t 658.566 ms\n \t 528.313 ms\n \t 946.492 ms\n \t Automatic redundancy enabled\n \t 1.465 ms\n \t 0.705 ms\n \t 4.004 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592096","8.7.3. Extended Save Data",
"8.7.3. Extended Save Data\nThis section shows benchmark data for accessing the extended save data region on media having the same specifications and capacity as the SD Card packaged with the CTR system. Use this data as a reference. The benchmark data provided was collected using a sample device with its sole purpose being measurement testing, so there is no guarantee that processing will complete within the times presented.\n Note:\n The media used for measurements exhibited a 2.5% drop in performance when reading and a 5.2% drop when writing for the transfer rate of a 505 KB file versus that of a 504 KB file. However, the degree of degradation of performance and the file size at which performance degrades might differ depending on the media’s capacity, manufacturer, or other factors, even for media of the same class. Because there are variations in performance depending on the media being used, be sure to treat the data listed here as a source of reference.\n",0]
,["Pages/Page_55592064.html#PageId_55592097","8.7.3.1. Mounting",
"8.7.3.1. Mounting\nThe tables below show the time required, as measured on a test unit, to execute the MountExtSaveData function for mounting extended save data, the DeleteExtSaveData function for deleting it, the CreateExtSaveData function for creating it, the MountExtSaveData function for remounting it, and the Unmount function for unmounting it.\n Table 8-11. Time Required for Extended Save Data Operations\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t MountExtSaveData\n \t 40.844 ms\n \t 39.278 ms\n \t 42.136 ms\n \t DeleteExtSaveData\n \t 105.296 ms\n \t 77.204 ms\n \t 165.070 ms\n \t CreateExtSaveData\n \t 2346.382 ms\n \t 2055.237 ms\n \t 2468.994 ms\n \t MountExtSaveData\n \t 38.256 ms\n \t 36.832 ms\n \t 40.576 ms\n \t Unmount\n \t 1.842 ms\n \t 1.619 ms\n \t 2.319 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592098","8.7.3.2. Creating Files",
"8.7.3.2. Creating Files\nThe tables below show the time required, as measured on a test unit, for the TryCreateFile function to execute.\n Table 8-12. Time Required to Create Files in Extended Save Data\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryCreateFile\n \t 766.200 ms\n \t 363.559 ms\n \t 2125.721 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592099","8.7.3.3. Loading Files",
"8.7.3.3. Loading Files\nThe figures below show the results for batch-reading extended save data files 50 times. In short, TryInitialize → TryRead → Finalize was performed 50 times. The time required for the TryRead function’s processing was measured on a test unit.\n Figure 8-14. Extended Save Data File Read Rates (Average Values)\nTo minimize the impact of caching, the measurement program changed the number of files used based on the capacity. For capacities up to 8 MB, every read operation was to a different, unique file. For 16 MB capacity, the testing involved 30 files, and for 32 MB the testing involved 16 files.\nThe tables below show the time taken by operations other than those of the TryRead function.\n Table 8-13. Time Taken for Processes Other Than the TryRead Function\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t 14.098 ms\n \t 11.246 ms\n \t 21.259 ms\n \t Finalize\n \t 1.433 ms\n \t 1.050 ms\n \t 3.150 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592100","8.7.3.4. Writing Files",
"8.7.3.4. Writing Files\nThe figures below show the results, measured on a test unit, for the time required to execute the operations of the TryWrite and TryFlush functions when writing files to extended save data 50 times. In short, TryInitialize → TryWrite → TryFlush → Finalize was performed 50 times.\n Figure 8-15. Extended Save Data File Write Rates (Average Values)\nTo prevent cached data from skewing the results, the number of files used for writing was dependent on the media capacity. For capacities up to 8 MB, every write operation was to a different, unique file. For 16 MB capacity, the testing used 30 files and for 32 MB, the testing used 16 files.\nThe tables below show the time taken by the TryInitialize function and the Finalize function under the different measurement conditions.\n Table 8-14. Time Taken by the TryInitialize and Finalize Functions\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t 15.406 ms\n \t 12.528 ms\n \t 23.478 ms\n \t Finalize\n \t 1.451 ms\n \t 1.040 ms\n \t 3.259 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592101","8.7.3.5. Deleting Files",
"8.7.3.5. Deleting Files\nThe tables below show the time required, as measured on a test unit, for the TryDeleteFile function to delete files in extended save data.\n Table 8-15. Time Required to Delete Files in Extended Save Data\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryDeleteFile\n \t 196.322 ms\n \t 108.301 ms\n \t 470.286 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592102","8.7.4. ROM Archives (Downloadable Applications)",
"8.7.4. ROM Archives (Downloadable Applications)\nThe benchmark data presented in this section resulted from measuring access to downloadable application’s ROM archives on media with the same specifications and capacity as the SD Card bundled with the CTR system. Because the benchmark data was collected using a sample device with its sole purpose being measurement testing, there is no guarantee that processing will complete within the times presented.\n",0]
,["Pages/Page_55592064.html#PageId_55592103","8.7.4.1. Batch Read",
"8.7.4.1. Batch Read\nThe figures below show the time required by the TryRead function to batch-read data of the specified size 50 times. In short, TryInitialize → TryRead → Finalize was performed 50 times. The green line shows the results obtained for downloadable applications. For reference, the blue dashed line shows the measurement results obtained for a development card (set to “Fast”).\n Figure 8-16. Downloadable Applications Transfer Rates (Batch Read)\nTo prevent cached data from skewing the results, we attempted to only read unique files whenever possible.\nThe tables below show the time taken by operations other than those of the TryRead function.\n Table 8-16. Downloadable Applications Time Required for Operations Other Than the TryRead Function\n \t Process\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t 0.046 ms\n \t 0.022 ms\n \t 0.220 ms\n \t Finalize\n \t 0.006 ms\n \t 0.002 ms\n \t 0.023 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592104","8.7.4.2. Stream Reads",
"8.7.4.2. Stream Reads\nThe figures below show the time required by the TryRead function to read files having the same size 50 times while varying the buffer size. A test unit measured the time required to execute the TryRead function. The green line shows the results obtained for downloadable applications. For reference, the blue dashed line shows the measurement results obtained for a development card (set to “Fast”).\nTo prevent cached data from skewing the results, we attempted to read only unique 8-MB files not stored in the cache.\n Figure 8-17. Downloadable Applications Transfer Rates (Stream Read)\n",0]
,["Pages/Page_55592064.html#PageId_55592105","8.7.5. Save Data (Downloadable Applications)",
"8.7.5. Save Data (Downloadable Applications)\nThe benchmark data presented in this section resulted from measuring access to a downloadable application's backup region on media with the same specifications and capacity as the SD Card bundled with the CTR system. Use this data as a reference. Because the benchmark data was collected using a sample device with its sole purpose being measurement testing, there is no guarantee that processing will complete within the times presented.\n",0]
,["Pages/Page_55592064.html#PageId_55592106","8.7.5.1. Read",
"8.7.5.1. Read\nThe figures below show the results for batch-reading the specified size of save data written to a downloadable application’s save data backup region set to 512 KB, with this operation performed a total of 50 times. In short, TryInitialize → TryRead → Finalize was performed 50 times. The time required for the TryRead function to execute was measured on a test unit. The red line shows results obtained with no automatic redundancy, and the blue line shows results when automatic redundancy was enabled.\n Figure 8-18. Downloadable Applications Save Data Read Rates (Average Values)\nTo minimize the impact of caching, the measurement program changed the number of files used based on the capacity. When reading files at the 4KB mark or less, all the files read were different, but for larger sizes, there was the possibility that caching would have an impact. Also note that the effect of caching changes when reading immediately after writing or when reading the same file more than once.\nThe tables below show the time taken by operations other than those of the TryRead function.\n Table 8-17. Downloadable Applications Time Required for Operations Other Than the TryRead Function (Save Data)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No automatic redundancy\n \t 1.259 ms\n \t 0.715 ms\n \t 7.613 ms\n \t Automatic redundancy enabled\n \t 1.242 ms\n \t 0.714 ms\n \t 5.633 ms\n \t Finalize\n \t No automatic redundancy\n \t 0.492 ms\n \t 0.349 ms\n \t 2.099 ms\n \t Automatic redundancy enabled\n \t 0.398 ms\n \t 0.278 ms\n \t 2.015 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592107","8.7.5.2. Writing",
"8.7.5.2. Writing\nThis section shows the results, measured on a test unit, for the time required to execute the TryWrite function and the TryFlush function’s operations when writing save data 50 times. In short, TryInitialize → TryWrite → TryFlush → Finalize was performed 50 times. The downloadable application backup memory size was 512 KB. Benchmarks were recorded with two types of save data: data saved with and without automatic redundancy enabled. When automatic redundancy was enabled, the measurement also included the operations of the CommitSaveData function.\n",0]
,["Pages/Page_55592064.html#Anchor_55592107_二重化なし","No Automatic Redundancy",
"No Automatic Redundancy\nThe figures below show the benchmark results for a save data region with no automatic redundancy. The green line shows results obtained when a new file was created (“No file” below), and the blue line shows results when the data overwrote a file of the same size (“Overwrite same size” below).\n Figure 8-19. Downloadable Applications Save Data Write Rates (No Automatic Redundancy)\nThere is almost no difference in the time required by the TryWrite function, regardless of whether a file is created or overwritten. The difference in processing speed (of approximately 200 ms) is due to the time taken by the TryFlush function.\nThe tables below show the time taken by the TryInitialize function and the Finalize function under the different measurement conditions.\n Table 8-18. Downloadable Applications Time Taken by the TryInitialize and Finalize Functions (No Automatic Redundancy)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No file\n \t 198.837 ms\n \t 55.597 ms\n \t 304.459 ms\n \t Overwrite file of the same size\n \t 1.232 ms\n \t 0.720 ms\n \t 4.068 ms\n \t Finalize\n \t No file\n \t 0.467 ms\n \t 0.333 ms\n \t 2.152 ms\n \t Overwrite file of the same size\n \t 0.466 ms\n \t 0.337 ms\n \t 2.134 ms\n The TryInitialize function takes more time when a file is created than when an existing file is overwritten.\n",0]
,["Pages/Page_55592064.html#Anchor_55592107_二重化あり","Automatic Redundancy Enabled",
"Automatic Redundancy Enabled\nThe figures below show the benchmark results for a save data region with automatic redundancy enabled. Because automatic redundancy is enabled, the total time includes the operations of the CommitSaveData function in addition to those of the TryWrite and TryFlush functions. The green line shows results obtained when a new file was created (“No file” below), and the blue line shows results when the data overwrote a file of the same size (“Overwrite same size” below).\n Figure 8-20. Downloadable Applications Save Data Write Rates (Automatic Redundancy Enabled)\nThe tables below show the time taken by the TryInitialize function and the Finalize function under the different measurement conditions.\n Table 8-19. Downloadable Applications Time Taken by the TryInitialize and Finalize Functions (Automatic Redundancy Enabled)\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryInitialize\n \t No file\n \t 1.256 ms\n \t 0.810 ms\n \t 3.412 ms\n \t Overwrite file of the same size\n \t 1.535 ms\n \t 0.662 ms\n \t 8.886 ms\n \t Finalize\n \t No file\n \t 0.396 ms\n \t 0.260 ms\n \t 2.002 ms\n \t Overwrite file of the same size\n \t 0.385 ms\n \t 0.260 ms\n \t 2.033 ms\n When a file was overwritten as opposed to created, some processes were dramatically slower. Otherwise, the results showed almost no differences under the two measurement conditions.\nThe figures below show the difference in transfer rate with and without automatic redundancy. The red line shows results obtained with no automatic redundancy, and the blue line shows results when automatic redundancy was enabled. In both cases, the measurement condition was “No file.”\n Figure 8-21. Downloadable Applications Save Data Write Rates (Comparison)\nIf your application supports the auto-save feature, Nintendo recommends against using a large amount of save data, because there is only a limited amount of time for finishing this process before the system powers off.\n",0]
,["Pages/Page_55592064.html#PageId_55592108","8.7.5.3. Mounting",
"8.7.5.3. Mounting\nThe tables below show the time required, as measured on a test unit, for the following functions: FormatSaveData, MountSaveData, and Unmount. The backup memory size for the downloadable application was 512 KB.\n Table 8-20. Downloadable Applications Time Required for Operations Related to Mounting Save Data\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t FormatSaveData\n \t No automatic redundancy (right after importing)\n \t 430.314 ms\n \t 360.571 ms\n \t 490.037 ms\n \t No automatic redundancy\n \t 436.077 ms\n \t 428.289 ms\n \t 446.790 ms\n \t Automatic redundancy enabled (right after importing)\n \t 206.321 ms\n \t 181.559 ms\n \t 253.866 ms\n \t Automatic redundancy enabled\n \t 197.499 ms\n \t 180.707 ms\n \t 215.180 ms\n \t MountSaveData\n \t No automatic redundancy\n \t 24.079 ms\n \t 22.886 ms\n \t 26.050 ms\n \t Automatic redundancy enabled\n \t 24.674 ms\n \t 22.770 ms\n \t 25.738 ms\n \t Unmount\n \t No automatic redundancy\n \t 1.694 ms\n \t 1.298 ms\n \t 3.496 ms\n \t Automatic redundancy enabled\n \t 1.338 ms\n \t 0.895 ms\n \t 2.622 ms\n For the measurement, the state immediately after importing is reproduced by deleting the backup region acted on by the SaveDataFiler function.\n",0]
,["Pages/Page_55592064.html#PageId_55592109","8.7.5.4. Deleting Files",
"8.7.5.4. Deleting Files\nThe tables below show the time required, as measured on a test unit, for the TryDeleteFile function. The backup memory size for the downloadable application was 512 KB.\n Table 8-21. Downloadable Applications Time Required for the TryDeleteFile Function\n \t Process\n \t Measurement Conditions\n \t Average\n \t Best Score\n \t Worst Score\n \t TryDeleteFile\n \t No automatic redundancy\n \t 171.093 ms\n \t 55.857 ms\n \t 299.145 ms\n \t Automatic redundancy enabled\n \t 1.546 ms\n \t 0.712 ms\n \t 4.581 ms\n",0]
,["Pages/Page_55592064.html#PageId_55592114","8.7.6. Comparison of CTR Flash Cards and CARD2",
"8.7.6. Comparison of CTR Flash Cards and CARD2\nCARD2 performance changes are shown below in comparison to the CTR flash card.\n",0]
,["Pages/Page_55592064.html#PageId_55592115","8.7.6.1. ROM Archive",
"8.7.6.1. ROM Archive\nThe performance between CTR flash cards and CARD2 is approximately equal.\n",0]
,["Pages/Page_55592064.html#PageId_55592116","8.7.6.2. Save Data",
"8.7.6.2. Save Data\n",0]
,["Pages/Page_55592064.html#Anchor_55592116_二重化なし","No Automatic Redundancy",
"No Automatic Redundancy\nCARD2 file read performance increased by a maximum factor of approximately 5x.\nFile write performance increased by a maximum factor of approximately 30x.\n",0]
,["Pages/Page_55592064.html#Anchor_55592116_二重化あり","Automatic Redundancy Enabled",
"Automatic Redundancy Enabled\nCARD2 file read performance increased by a maximum factor of approximately 5x.\nFile write performance increased by a maximum factor of approximately 33x.\nCONFIDENTIAL\n",0]
,["Pages/Page_55594375.html","2. Extended Applications",
"2. Extended Applications\nThis chapter provides information about extended applications that can take advantage of the improved hardware of SNAKE.\nAn application that runs on CTR-equivalent hardware even on SNAKE is referred to as a standard application.\nCTR game software applications that have already been released to the market are considered standard applications.\n",0]
,["Pages/Page_55594375.html#PageId_55594453","2.1. Enhanced Features in Extended Applications (Important)",
"2.1. Enhanced Features in Extended Applications (Important)\nWhen an extended application is run on SNAKE, the CPU performance and amount of main memory available is increased as follows.\n Table 2-1. Enhanced Features in Extended Applications\n \tFeature \tStandard Application \tExtended Application\n \tCPU Clock \t268 MHz \t804 MHz\n \tL2 Cache \tNone \t2 MB\n \tMain Memory \t64 MB \t124 MB\n Due to the large size of the CPU L2 cache that is enabled in extended applications, the nngxUpdateBuffer function and other functions that are called when using the results of CPU memory operations in other components (such as the GPU) can have an effect on performance.\nFor more information, see Section 3.4.10. Differences Between nngxUpdateBuffer and nngxUpdateBufferLight (Important).\n",0]
,["Pages/Page_55594375.html#PageId_55594502","2.2. Running Extended Applications on CTR",
"2.2. Running Extended Applications on CTR\nAny application built as an extended application can be run on CTR without any changes.\nHowever, applications only run with the same hardware performance as a standard application.\n",0]
,["Pages/Page_55594375.html#PageId_55594600","2.3. Checking the Operating Environment (Important)",
"2.3. Checking the Operating Environment (Important)\nFunctions are provided to determine the environment that an application is running in so that the application can be adjusted based on the operating environment.\n Table 2-2. Functions for Determining the Operating Environment in an Application\n \tFunction \tDescription\n \t nn::os::IsRunOnSnake \tReturns whether the application is running on SNAKE hardware.\n \t nn::os::IsRunningAsExtApplication \tReturns whether the application is running as an extended application.\n Nintendo recommends calling these functions when the application starts and caching the return value. The nn::os::IsRunningAsExtApplication function in particular can have a negative effect on performance when called frequently.\n",0]
,["Pages/Page_55594375.html#PageId_55594479","2.4. Standard Applications With Reduced Performance Only on SNAKE",
"2.4. Standard Applications With Reduced Performance Only on SNAKE\nDue to differences in scheduling and the configuration of threads running on the system core, some standard applications might experience performance problems only when running on SNAKE hardware.\nIn such cases, the performance problems can typically be resolved by building the application as an extended application.\nCONFIDENTIAL\n",0]
];
for (var x = 0; x < Search.additionalData.length; x++) Search.indexData.push(Search.additionalData[x]);
